# -*- encoding: utf-8 -*-
#
# Copyright Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base


RESOURCE = "jobs_events"


def list(context, sequence, limit=10, offset=0):
    params = {"limit": limit, "offset": offset}
    uri = "%s/%s/%s" % (context.dci_cs_api, RESOURCE, sequence)
    return context.session.get(uri, params=params)


def iter(context, sequence, limit=10):
    """Iter to list all the jobs events."""
    params = {"limit": limit, "offset": 0}
    uri = "%s/%s/%s" % (context.dci_cs_api, RESOURCE, sequence)

    while True:
        j = context.session.get(uri, params=params).json()
        if len(j["jobs_events"]):
            for i in j["jobs_events"]:
                yield i
        else:
            break
        params["offset"] += params["limit"]


def delete(context, sequence):
    """Delete jobs events from a given sequence"""
    uri = "%s/%s/%s" % (context.dci_cs_api, RESOURCE, sequence)
    return context.session.delete(uri)


def get_sequence(context):
    uri = "%s/%s/sequence" % (context.dci_cs_api, RESOURCE)
    return context.session.get(uri)


def update_sequence(context, etag, sequence):
    uri = "%s/%s/sequence" % (context.dci_cs_api, RESOURCE)
    return context.session.put(
        uri,
        timeout=base.HTTP_TIMEOUT,
        headers={"If-match": etag},
        json={"sequence": sequence},
    )



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import job
from dciclient.v1.api import file


def list(context, args):
    params = {k: getattr(args, k) for k in ["sort", "limit", "offset", "where"]}
    return job.list_files(context, id=args.job_id, **params)


def show(context, args):
    return file.content(context, id=args.id)


def delete(context, args):
    return file.delete(context, id=args.id)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import file as dci_file
from dciclient.v1.api import job


def list(context, args):
    params = {k: getattr(args, k) for k in ["sort", "limit", "offset", "where"]}
    params["embed"] = "topic,remoteci,team"
    return job.list(context, **params)


def show(context, args):
    return job.get(context, id=args.id)


def delete(context, args):
    return job.delete(context, id=args.id, etag=args.etag)


def list_results(context, args):
    params = {k: getattr(args, k) for k in ["sort", "limit", "offset", "id"]}
    return job.list_results(context, **params)


def attach_issue(context, args):
    return job.attach_issue(context, id=args.id, url=args.url)


def unattach_issue(context, args):
    return job.unattach_issue(context, id=args.id, issue_id=args.issue_id)


def list_issues(context, args):
    params = {k: getattr(args, k) for k in ["sort", "limit", "offset", "id", "where"]}
    return job.list_issues(context, **params)


def output(context, args):
    result = job.list_jobstates(context, id=args.id, sort="created_at")
    jobstates = result.json()["jobstates"]

    res = []
    for js in jobstates:
        f_l = job.list_files(
            context, id=args.id, where="jobstate_id:" + js["id"], sort="created_at"
        )
        for f in f_l.json()["files"]:
            res.append(dci_file.content(context, id=f["id"]).text)
    return res


def list_tests(context, args):
    params = {k: getattr(args, k) for k in ["sort", "limit", "offset", "id", "where"]}
    return job.list_tests(context, **params)


def add_tag(context, args):
    return job.add_tag(context, id=args.id, name=args.name)


def delete_tag(context, args):
    return job.delete_tag(context, id=args.id, tag_id=args.tag_id)


def list_tags(context, args):
    return job.list_tags(context, args.id)


def file_upload(context, args):
    params = {
        k: getattr(args, k)
        for k in ["job_id", "name", "file_path", "jobstate_id", "test_id", "mime"]
    }
    return dci_file.create_with_stream(context, **params)


def file_download(context, args):
    params = {k: getattr(args, k) for k in ["id", "file_id", "target"]}
    dci_file.download(context, **params)


def file_show(context, args):
    return dci_file.get(context, id=args.file_id)


def file_list(context, args):
    params = {k: getattr(args, k) for k in ["sort", "limit", "offset", "id", "where"]}
    return job.list_files(context, **params)


def file_delete(context, args):
    dci_file.delete(context, id=args.file_id)
    return dci_file.delete(context, id=args.file_id)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals


def test_show(runner, file_id):
    result = runner.invoke_raw(["file-show", file_id])
    assert "testsuite errors" in result.text


def test_list(runner, job_id):
    files = runner.invoke(["file-list", job_id])["files"]
    assert len(files)
    assert "res_junit.xml" in [i["name"] for i in files]


def test_delete(runner, file_id):
    result = runner.invoke_raw(["file-delete", file_id])
    assert result.status_code == 204


def test_where_on_list(runner, job_id):
    assert (
        runner.invoke(["file-list", job_id, "--where", "size:785"])["_meta"]["count"]
        == 1
    )



# -*- encoding: utf-8 -*-
#
# Copyright Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from dciclient.v1.api import analytic


def list(context, args):
    return analytic.list(context, job_id=args.job_id)


def create(context, args):
    params = {k: getattr(args, k) for k in ["job_id", "name", "type", "url", "data"]}
    return analytic.create(context, **params)


def show(context, args):
    return analytic.get(context, id=args.id, job_id=args.job_id)


def update(context, args):
    params = {
        k: getattr(args, k)
        for k in ["id", "etag", "job_id", "name", "type", "url", "data"]
    }
    return analytic.update(context, **params)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals


def test_list(runner, product_id):
    topic = runner.invoke(
        ["topic-create", "--name", "osp", "--product-id", product_id]
    )["topic"]
    teams = runner.invoke(["team-list"])["teams"]
    team_id = teams[0]["id"]

    runner.invoke(["topic-attach-team", topic["id"], "--team-id", team_id])

    runner.invoke(
        [
            "component-create",
            "--name",
            "foo",
            "--type",
            "bar",
            "--topic-id",
            topic["id"],
        ]
    )
    runner.invoke(
        [
            "component-create",
            "--name",
            "bar",
            "--type",
            "bar2",
            "--topic-id",
            topic["id"],
        ]
    )
    components = runner.invoke(["component-list", "--topic-id", topic["id"]])[
        "components"
    ]

    assert len(components) == 2
    assert components[0]["name"] == "bar"
    assert components[1]["name"] == "foo"


def test_create(runner, product_id):
    topic = runner.invoke(
        ["topic-create", "--name", "osp", "--product-id", product_id]
    )["topic"]

    component = runner.invoke(
        [
            "component-create",
            "--name",
            "foo",
            "--type",
            "foobar",
            "--topic-id",
            topic["id"],
        ]
    )["component"]
    assert component["name"] == "foo"


def test_create_inactive(runner, product_id):
    topic = runner.invoke(
        ["topic-create", "--name", "osp", "--product-id", product_id]
    )["topic"]

    component = runner.invoke(
        [
            "component-create",
            "--name",
            "foo",
            "--type",
            "foobar",
            "--topic-id",
            topic["id"],
            "--no-active",
        ]
    )["component"]
    assert component["state"] == "inactive"


def test_delete(runner, product_id):
    topic = runner.invoke(
        ["topic-create", "--name", "osp", "--product-id", product_id]
    )["topic"]
    component = runner.invoke(
        [
            "component-create",
            "--name",
            "foo",
            "--type",
            "bar",
            "--topic-id",
            topic["id"],
        ]
    )["component"]

    result = runner.invoke_raw(["component-delete", component["id"]])

    assert result.status_code == 204


def test_show(runner, product_id):
    topic = runner.invoke(
        ["topic-create", "--name", "osp", "--product-id", product_id]
    )["topic"]
    assert topic["name"] == "osp"

    component = runner.invoke(
        [
            "component-create",
            "--name",
            "foo",
            "--type",
            "bar",
            "--topic-id",
            topic["id"],
        ]
    )["component"]

    result = runner.invoke(["component-show", component["id"]])

    assert result["component"]["name"] == "foo"


def test_file_support(runner, tmpdir, product_id):
    td = tmpdir
    p = td.join("hello.txt")
    p.write("content")
    topic = runner.invoke(
        ["topic-create", "--name", "osp", "--product-id", product_id]
    )["topic"]

    component = runner.invoke(
        [
            "component-create",
            "--name",
            "foo",
            "--type",
            "foobar",
            "--topic-id",
            topic["id"],
        ]
    )["component"]

    # upload
    new_f = runner.invoke(
        ["component-file-upload", component["id"], "--path", p.strpath]
    )["component_file"]
    assert new_f["size"] == 7

    # show
    new_f = runner.invoke(
        ["component-file-show", component["id"], "--file-id", new_f["id"]]
    )["component_file"]
    assert new_f["size"] == 7

    # download
    runner.invoke_raw(
        [
            "component-file-download",
            component["id"],
            "--file-id",
            new_f["id"],
            "--target",
            td.strpath + "/my_file",
        ]
    )
    assert open(td.strpath + "/my_file", "r").read() == "content"

    # list
    my_list = runner.invoke(["component-file-list", component["id"]])["component_files"]
    assert len(my_list) == 1
    assert my_list[0]["size"] == 7

    # delete
    runner.invoke_raw(
        ["component-file-delete", component["id"], "--file-id", new_f["id"]]
    )
    result = runner.invoke_raw(
        ["component-file-show", component["id"], "--file-id", new_f["id"]]
    )
    assert result.status_code == 404


def test_update_active(runner, product_id):
    topic = runner.invoke(
        ["topic-create", "--name", "osp", "--product-id", product_id]
    )["topic"]

    component = runner.invoke(
        [
            "component-create",
            "--name",
            "foo",
            "--type",
            "bar",
            "--topic-id",
            topic["id"],
        ]
    )["component"]

    assert component["state"] == "active"

    result = runner.invoke(["component-update", component["id"], "--no-active"])
    assert result["component"]["state"] == "inactive"

    result = runner.invoke(["component-update", component["id"], "--active"])
    assert result["component"]["state"] == "active"


def test_where_on_list(runner, product_id):
    topic = runner.invoke(
        ["topic-create", "--name", "osp", "--product-id", product_id]
    )["topic"]
    teams = runner.invoke(["team-list"])["teams"]
    team_id = teams[0]["id"]

    runner.invoke(["topic-attach-team", topic["id"], "--team-id", team_id])

    runner.invoke(
        [
            "component-create",
            "--name",
            "foo",
            "--type",
            "bar",
            "--topic-id",
            topic["id"],
        ]
    )
    runner.invoke(
        [
            "component-create",
            "--name",
            "bar",
            "--type",
            "bar2",
            "--topic-id",
            topic["id"],
        ]
    )

    assert (
        runner.invoke(
            ["component-list", "--topic-id", topic["id"], "--where", "type:bar2"]
        )["_meta"]["count"]
        == 1
    )



from dciclient.v1 import utils


def print_response(response, format, verbose):
    if response.status_code == 204:
        utils.print_json({"status_code": 204, "message": "resource deleted."})
    else:
        utils.format_output(response, format, verbose=verbose)



# -*- encoding: utf-8 -*-
#
# Copyright Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


def test_create(runner, job_id):
    analytic = runner.invoke(
        [
            "analytic-create",
            "--job-id",
            job_id,
            "--name",
            "bug1",
            "--type",
            "infrastructure",
            "--url",
            "http://bugzilla/1",
            "--data",
            '{"root_cause": {}}',
        ]
    )["analytic"]
    assert analytic["name"] == "bug1"
    assert analytic["job_id"] == job_id
    assert analytic["type"] == "infrastructure"
    assert analytic["url"] == "http://bugzilla/1"
    assert analytic["data"] == {"root_cause": {}}


def test_show(runner, job_id):
    analytic = runner.invoke(
        [
            "analytic-create",
            "--job-id",
            job_id,
            "--name",
            "bug1",
            "--type",
            "infrastructure",
            "--url",
            "http://bugzilla/1",
            "--data",
            '{"root_cause": {}}',
        ]
    )["analytic"]
    assert analytic["name"] == "bug1"

    analytic = runner.invoke(["analytic-show", "--job-id", job_id, analytic["id"]])[
        "analytic"
    ]
    assert analytic["name"] == "bug1"
    assert analytic["type"] == "infrastructure"
    assert analytic["url"] == "http://bugzilla/1"
    assert analytic["data"] == {"root_cause": {}}


def test_list(runner, job_id):
    analytic = runner.invoke(
        [
            "analytic-create",
            "--job-id",
            job_id,
            "--name",
            "bug1",
            "--type",
            "infrastructure",
            "--url",
            "http://bugzilla/1",
            "--data",
            '{"root_cause": {}}',
        ]
    )["analytic"]
    assert analytic["name"] == "bug1"

    analytic = runner.invoke(
        [
            "analytic-create",
            "--job-id",
            job_id,
            "--name",
            "bug2",
            "--type",
            "infrastructure",
            "--url",
            "http://bugzilla/2",
            "--data",
            '{"root_cause": {}}',
        ]
    )["analytic"]
    assert analytic["name"] == "bug2"

    all_anc = runner.invoke(["analytic-list", "--job-id", job_id])
    assert len(all_anc["analytics"]) == 2


def test_update(runner, job_id, dci_context):
    analytic = runner.invoke(
        [
            "analytic-create",
            "--job-id",
            job_id,
            "--name",
            "bug1",
            "--type",
            "infrastructure",
            "--url",
            "http://bugzilla/1",
            "--data",
            '{"root_cause": {}}',
        ]
    )["analytic"]
    assert analytic["name"] == "bug1"

    analytic = runner.invoke(
        [
            "analytic-update",
            analytic["id"],
            "--job-id",
            job_id,
            "--etag",
            analytic["etag"],
            "--name",
            "bug2",
            "--type",
            "infrastructure",
            "--url",
            "http://bugzilla/2",
            "--data",
            '{"root_cause": {}}',
        ]
    )["analytic"]
    assert analytic["name"] == "bug2"
    assert analytic["url"] == "http://bugzilla/2"



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base
from dciclient.v1 import utils

import io
import os

RESOURCE = "files"


def create(
    context,
    name,
    content=None,
    file_path=None,
    mime="text/plain",
    jobstate_id=None,
    md5=None,
    job_id=None,
    test_id=None,
):
    """Method to create a file on the Control-Server

    This method allows one to upload a file to the Control-Server. The file
    to be uploaded can be specified in two different ways either by specifying
    its content directly or or by specifying the file_path where the file is
    located.

    content can be in the form of: string, bytes or a file-descriptor.

    """

    if content and file_path:
        raise Exception("content and file_path are mutually exclusive")
    elif not content and not file_path:
        raise Exception("At least one of content or file_path must be specified")

    headers = {
        "DCI-NAME": name,
        "DCI-MIME": mime,
        "DCI-JOBSTATE-ID": jobstate_id,
        "DCI-MD5": md5,
        "DCI-JOB-ID": job_id,
        "DCI-TEST-ID": test_id,
    }
    headers = utils.sanitize_kwargs(**headers)
    uri = "%s/%s" % (context.dci_cs_api, RESOURCE)

    if content:
        if not hasattr(content, "read"):
            if not isinstance(content, bytes):
                content = content.encode("utf-8")
            content = io.BytesIO(content)
        return context.session.post(uri, headers=headers, data=content)
    else:
        if not os.path.exists(file_path):
            raise FileErrorException()
        with open(file_path, "rb") as f:
            return context.session.post(uri, headers=headers, data=f)


# TODO(spredzy): Remove this method once all the party using this method has
#                moved to create()
def create_with_stream(
    context,
    name,
    file_path,
    mime="text/plain",
    jobstate_id=None,
    md5=None,
    job_id=None,
    test_id=None,
):
    return create(
        context,
        name,
        file_path=file_path,
        mime=mime,
        jobstate_id=jobstate_id,
        md5=md5,
        job_id=job_id,
        test_id=test_id,
    )


def get(context, id, **kwargs):
    return base.get(context, RESOURCE, id=id, **kwargs)


def list(context, **kwargs):
    return base.list(context, RESOURCE, **kwargs)


def iter(context, **kwargs):
    return base.iter(context, RESOURCE, **kwargs)


def delete(context, id):
    return base.delete(context, RESOURCE, id=id)


def content(context, id):
    uri = "%s/%s/%s/content" % (context.dci_cs_api, RESOURCE, id)
    r = context.session.get(uri)
    return r


def download(context, id, file_id, target):
    uri = "%s/files/%s/content" % (context.dci_cs_api, file_id)
    return base.download(context, uri, target)


class FileErrorException(Exception):
    def __init__(self, *args, **kwargs):
        super(FileErrorException, self).__init__(self, *args, **kwargs)



# -*- encoding: utf-8 -*-
#
# Copyright Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys
from mock import patch
from pytest import raises
from dciclient.v1.shell_commands.cli import parse_arguments, _default_dci_cs_url
from dciclient.version import __version__


@patch("sys.exit")
def test_parse_arguments_version(exit_function, capsys):
    parse_arguments(["--version", "user-list"])
    captured = capsys.readouterr()
    # Note(hguemar): argparse behaviour here changed on py3k
    if sys.version_info > (3, 0):
        captured = captured.out
    else:
        captured = captured.err
    assert captured == "dcictl {}\n".format(__version__)
    assert exit_function.called


def test_parse_arguments_format():
    args = parse_arguments(["--format", "json", "user-list"])
    assert args.format == "json"
    args = parse_arguments(["--format", "csv", "user-list"])
    assert args.format == "csv"
    args = parse_arguments(["--format", "tsv", "user-list"])
    assert args.format == "tsv"
    args = parse_arguments(["user-list"])
    assert args.format == "table"


def test_parse_arguments_dci_login():
    args = parse_arguments(["--dci-login", "foo", "user-list"], {"DCI_LOGIN": "foo"})
    assert args.dci_login == "foo"


def test_parse_arguments_dci_login_from_env():
    args = parse_arguments(["user-list"], {"DCI_LOGIN": "foo"})
    assert args.dci_login == "foo"


def test_parse_arguments_dci_login_overload_from_env():
    args = parse_arguments(["--dci-login", "bar", "user-list"], {"DCI_LOGIN": "foo"})
    assert args.dci_login == "bar"


def test_parse_arguments_dci_password():
    args = parse_arguments(["--dci-password", "foo", "user-list"])
    assert args.dci_password == "foo"


def test_parse_arguments_dci_password_from_env():
    args = parse_arguments(["user-list"], {"DCI_PASSWORD": "foo"})
    assert args.dci_password == "foo"


def test_parse_arguments_dci_password_overload_from_env():
    args = parse_arguments(
        ["--dci-password", "bar", "user-list"], {"DCI_PASSWORD": "foo"}
    )
    assert args.dci_password == "bar"


def test_parse_arguments_dci_client_id():
    args = parse_arguments(["--dci-client-id", "foo", "user-list"])
    assert args.dci_client_id == "foo"


def test_parse_arguments_dci_client_id_from_env():
    args = parse_arguments(["user-list"], {"DCI_CLIENT_ID": "foo"})
    assert args.dci_client_id == "foo"


def test_parse_arguments_dci_client_id_overload_from_env():
    args = parse_arguments(
        ["--dci-client-id", "bar", "user-list"], {"DCI_CLIENT_ID": "foo"}
    )
    assert args.dci_client_id == "bar"


def test_parse_arguments_dci_api_secret():
    args = parse_arguments(["--dci-api-secret", "foo", "user-list"])
    assert args.dci_api_secret == "foo"


def test_parse_arguments_dci_api_secret_from_env():
    args = parse_arguments(["user-list"], {"DCI_API_SECRET": "foo"})
    assert args.dci_api_secret == "foo"


def test_parse_arguments_dci_api_secret_overload_from_env():
    args = parse_arguments(
        ["--dci-api-secret", "bar", "user-list"], {"DCI_API_SECRET": "foo"}
    )
    assert args.dci_api_secret == "bar"


def test_parse_arguments_dci_cs_url_default():
    args = parse_arguments(["user-list"])
    assert args.dci_cs_url == _default_dci_cs_url


def test_parse_arguments_dci_cs_url():
    args = parse_arguments(["--dci-cs-url", "foo", "user-list"])
    assert args.dci_cs_url == "foo"


def test_parse_arguments_dci_cs_url_from_env():
    args = parse_arguments(["user-list"], {"DCI_CS_URL": "foo"})
    assert args.dci_cs_url == "foo"


def test_parse_arguments_dci_cs_url_overload_from_env():
    args = parse_arguments(["--dci-cs-url", "bar", "user-list"], {"DCI_CS_URL": "foo"})
    assert args.dci_cs_url == "bar"


# fear test
@patch("sys.exit")
def test_parse_arguments_user_create_mutually_exclusive_boolean_flags(exit_function):
    with raises(TypeError):
        parse_arguments(
            [
                "user-create",
                "--name",
                "foo",
                "--password",
                "bar",
                "--email",
                "foo@foo.bar",
                "--active",
                "--no-active",
            ]
        )
        assert exit_function.called


def test_parse_arguments_sso():
    args = parse_arguments(
        [
            "--sso-url",
            "https://sso.redhat.com",
            "--sso-username",
            "dci",
            "--sso-password",
            "dci",
            "user-list",
        ],
        {},
    )

    assert args.sso_url == "https://sso.redhat.com"
    assert args.sso_username == "dci"
    assert args.sso_password == "dci"
    assert args.sso_token is None
    assert args.refresh_sso_token is False


def test_parse_arguments_sso_env():
    args = parse_arguments(
        ["user-list"],
        {
            "SSO_URL": "https://sso.redhat.com",
            "SSO_USERNAME": "sso",
            "SSO_PASSWORD": "sso",
        },
    )

    assert args.sso_url == "https://sso.redhat.com"
    assert args.sso_username == "sso"
    assert args.sso_password == "sso"
    assert args.sso_token is None
    assert args.refresh_sso_token is False


def test_parse_arguments_sso_token():
    args = parse_arguments(
        ["--sso-token", "abc", "--refresh-sso-token", "user-list"], {}
    )

    assert args.refresh_sso_token
    assert args.sso_token == "abc"


def test_parse_arguments_sso_token_env():
    args = parse_arguments(["user-list"], {"SSO_TOKEN": "efg"})

    assert args.sso_token == "efg"
    assert args.refresh_sso_token is False


def test_verbose():
    args = parse_arguments(
        [
            "user-create",
            "--name",
            "toto",
            "--password",
            "toto",
            "--email",
            "toto@example.org",
        ]
    )
    assert args.verbose is False

    args = parse_arguments(
        [
            "user-create",
            "--verbose",
            "--name",
            "toto",
            "--password",
            "toto",
            "--email",
            "toto@example.org",
        ]
    )
    assert args.verbose is True

    args = parse_arguments(
        [
            "user-create",
            "--long",
            "--name",
            "toto",
            "--password",
            "toto",
            "--email",
            "toto@example.org",
        ]
    )
    assert args.verbose is True



# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1 import utils

import os

HTTP_TIMEOUT = 600


def create(context, resource, **kwargs):
    """Create a resource"""
    data = utils.sanitize_kwargs(**kwargs)
    uri = "%s/%s" % (context.dci_cs_api, resource)
    r = context.session.post(uri, timeout=HTTP_TIMEOUT, json=data)
    return r


def list(context, resource, **kwargs):
    """List all resources"""
    data = utils.sanitize_kwargs(**kwargs)
    id = data.pop("id", None)
    subresource = data.pop("subresource", None)

    if subresource:
        uri = "%s/%s/%s/%s" % (context.dci_cs_api, resource, id, subresource)
    else:
        uri = "%s/%s" % (context.dci_cs_api, resource)

    return context.session.get(uri, timeout=HTTP_TIMEOUT, params=data)


def iter(context, resource, **kwargs):
    """List all resources"""
    data = utils.sanitize_kwargs(**kwargs)
    id = data.pop("id", None)
    subresource = data.pop("subresource", None)
    data["limit"] = data.get("limit", 20)

    if subresource:
        uri = "%s/%s/%s/%s" % (context.dci_cs_api, resource, id, subresource)
        resource = subresource
    else:
        uri = "%s/%s" % (context.dci_cs_api, resource)

    data["offset"] = 0
    while True:
        j = context.session.get(uri, timeout=HTTP_TIMEOUT, params=data).json()
        if len(j[resource]):
            for i in j[resource]:
                yield i
        else:
            break
        data["offset"] += data["limit"]


def get(context, resource, **kwargs):
    """List a specific resource"""
    uri = "%s/%s/%s" % (context.dci_cs_api, resource, kwargs.pop("id"))
    r = context.session.get(uri, timeout=HTTP_TIMEOUT, params=kwargs)
    return r


def get_data(context, resource, **kwargs):
    """Retrieve data field from a resource"""

    url_suffix = ""
    if "keys" in kwargs and kwargs["keys"]:
        url_suffix = "/?keys=%s" % ",".join(kwargs.pop("keys"))

    uri = "%s/%s/%s/data%s" % (
        context.dci_cs_api,
        resource,
        kwargs.pop("id"),
        url_suffix,
    )

    r = context.session.get(uri, timeout=HTTP_TIMEOUT, params=kwargs)
    return r


def update(context, resource, **kwargs):
    """Update a specific resource"""
    etag = kwargs.pop("etag")
    id = kwargs.pop("id")
    data = utils.sanitize_kwargs(**kwargs)
    uri = "%s/%s/%s" % (context.dci_cs_api, resource, id)
    r = context.session.put(
        uri, timeout=HTTP_TIMEOUT, headers={"If-match": etag}, json=data
    )
    return r


def delete(context, resource, id, **kwargs):
    """Delete a specific resource"""

    etag = kwargs.pop("etag", None)
    id = id
    subresource = kwargs.pop("subresource", None)
    subresource_id = kwargs.pop("subresource_id", None)

    uri = "%s/%s/%s" % (context.dci_cs_api, resource, id)
    if subresource:
        uri = "%s/%s/%s" % (uri, subresource, subresource_id)

    r = context.session.delete(uri, timeout=HTTP_TIMEOUT, headers={"If-match": etag})
    return r


def purge(context, resource, **kwargs):
    """Purge resource type."""
    uri = "%s/%s/purge" % (context.dci_cs_api, resource)
    if "force" in kwargs and kwargs["force"]:
        r = context.session.post(uri, timeout=HTTP_TIMEOUT)
    else:
        r = context.session.get(uri, timeout=HTTP_TIMEOUT)
    return r


def download(context, uri, target):
    r = context.session.get(uri, stream=True, timeout=HTTP_TIMEOUT)
    r.raise_for_status()
    with open(target + ".part", "wb") as f:
        for chunk in r.iter_content(chunk_size=1024):
            if chunk:
                f.write(chunk)
    os.rename(target + ".part", target)
    return r



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import job

import pytest
import requests
import requests.exceptions


try:
    requests.get("http://google.com")
except requests.exceptions.ConnectionError:
    internet_cnx = False
else:
    internet_cnx = True


def test_list(
    runner, dci_context, dci_context_remoteci, team_user_id, remoteci_id, product_id,
):
    topic = runner.invoke(
        [
            "topic-create",
            "--name",
            "osp",
            "--component_types",
            "type_1",
            "--no-export-control",
            "--product-id",
            product_id,
        ]
    )["topic"]

    runner.invoke(["topic-attach-team", topic["id"], "--team-id", team_user_id])

    runner.invoke(
        [
            "component-create",
            "--name",
            "foo",
            "--type",
            "type_1",
            "--topic-id",
            topic["id"],
        ]
    )["component"]

    job.schedule(dci_context_remoteci, topic["id"])
    l_job = runner.invoke(["job-list"])
    assert len(l_job["jobs"]) == 1
    assert l_job["jobs"][0]["remoteci"]["id"] == remoteci_id
    assert l_job["jobs"][0]["topic"]["id"] == topic["id"]
    output = runner.invoke(["job-list"])
    assert output["jobs"][0]["topic"]["name"] == "osp"
    assert output["jobs"][0]["id"] == l_job["jobs"][0]["id"]

    l_job = runner.invoke(["job-list", "--where", "remoteci_id:" + remoteci_id])
    assert len(l_job["jobs"]) == 1


def test_list_as_remoteci(job, remoteci_id, runner_remoteci):
    l_job = runner_remoteci.invoke(["job-list"])
    assert len(l_job["jobs"]) == 1
    assert l_job["jobs"][0]["remoteci"]["id"] == remoteci_id
    assert l_job["jobs"][0]["topic"]["id"] == job["topic_id"]
    output = runner_remoteci.invoke(["job-list"])
    assert output["jobs"][0]["topic"]["name"] == "foo_topic"
    assert output["jobs"][0]["id"] == job["id"]

    l_job = runner_remoteci.invoke(
        ["job-list", "--where", "remoteci_id:" + remoteci_id]
    )
    assert len(l_job["jobs"]) == 1


def test_list_with_limit(runner, job_factory):
    for _ in range(6):
        job_factory()
    # test --limit XX
    l_job = runner.invoke(["job-list"])
    assert len(l_job["jobs"]) == 6
    l_job = runner.invoke(["job-list", "--limit", "1"])
    assert len(l_job["jobs"]) == 1


def test_delete(runner, job_id):
    l_job = runner.invoke(["job-show", job_id])
    l_job_etag = l_job["job"]["etag"]

    result = runner.invoke_raw(["job-delete", job_id, "--etag", l_job_etag])

    assert result.status_code == 204


def test_results(runner, job_id):
    result = runner.invoke(["job-results", job_id])["results"][0]

    assert result["filename"] == "res_junit.xml"


@pytest.mark.skipif(not internet_cnx, reason="internet connection required")
def test_attach_issue(runner, job_id):
    result = runner.invoke(["job-list-issue", job_id])["_meta"]["count"]
    assert result == 0

    issue = runner.invoke(
        [
            "job-attach-issue",
            job_id,
            "--url",
            "https://github.com/redhat-cip/dci-control-server/issues/2",
        ]
    )
    # NOTE(Goneri): until we fix the consistency issue with this endpoint:
    # https://softwarefactory-project.io/r/6863
    if "issue" in issue:
        issue = issue["issue"]
    else:
        issue["id"] = issue["issue-id"]
    result = runner.invoke(["job-list-issue", job_id])
    assert issue["id"] == result["issues"][0]["id"]


@pytest.mark.skipif(not internet_cnx, reason="internet connection required")
def test_unattach_issue(runner, job_id):
    result = runner.invoke(["job-list-issue", job_id])["_meta"]["count"]
    assert result == 0

    runner.invoke(
        [
            "job-attach-issue",
            job_id,
            "--url",
            "https://github.com/redhat-cip/dci-control-server/issues/2",
        ]
    )
    result = runner.invoke(["job-list-issue", job_id])
    res = result["_meta"]["count"]
    issue_id = result["issues"][0]["id"]
    assert res == 1

    runner.invoke_raw(["job-unattach-issue", job_id, "--issue-id", issue_id])
    result = runner.invoke(["job-list-issue", job_id])
    count = result["_meta"]["count"]
    assert count == 0


def test_job_output(runner, job_id):
    result = runner.invoke_raw(["job-output", job_id])
    assert result[0].startswith("pre-run")


def test_tags(runner, job_id):
    tags = runner.invoke(["job-list-tags", job_id])["tags"]
    assert len(tags) == 0
    tag = runner.invoke(["job-add-tag", job_id, "foo"])["tag"]
    tags = runner.invoke(["job-list-tags", job_id])["tags"]
    assert len(tags) == 1
    assert tags[0]["id"] == tag["id"]
    assert tags[0]["name"] == "foo"
    runner.invoke_raw(["job-delete-tag", job_id, tag["id"]])
    tags = runner.invoke(["job-list-tags", job_id])["tags"]
    assert len(tags) == 0


def test_file_support(runner, tmpdir, job_id):
    td = tmpdir
    p = td.join("hello.txt")
    p.write("content")

    # upload
    new_f = runner.invoke(
        [
            "job-upload-file",
            job_id,
            "--name",
            "test",
            "--mime",
            "application/octet-stream",
            "--path",
            p.strpath,
        ]
    )["file"]
    assert new_f["size"] == 7

    # show
    new_f = runner.invoke(["job-show-file", job_id, "--file-id", new_f["id"]])["file"]
    assert new_f["size"] == 7
    assert new_f["mime"] == "application/octet-stream"

    # download
    runner.invoke_raw(
        [
            "job-download-file",
            job_id,
            "--file-id",
            new_f["id"],
            "--target",
            td.strpath + "/my_file",
        ]
    )
    assert open(td.strpath + "/my_file", "r").read() == "content"

    # list
    my_list = runner.invoke(["job-list-file", job_id])["files"]
    assert len(my_list) == 3
    assert my_list[0]["size"] == 7

    # delete
    runner.invoke_raw(["job-delete-file", job_id, "--file-id", new_f["id"]])
    result = runner.invoke_raw(["job-show-file", job_id, "--file-id", new_f["id"]])
    assert result.status_code == 404


def test_file_support_as_remoteci(runner_remoteci, tmpdir, job_id):
    td = tmpdir
    p = td.join("remoteci.txt")
    content = u"remoteci content".encode("utf-8")
    p.write(content, "wb")

    my_original_list = runner_remoteci.invoke(["job-list-file", job_id])["files"]

    # upload
    new_f = runner_remoteci.invoke(
        ["job-upload-file", job_id, "--name", "testrci", "--path", p.strpath]
    )["file"]
    assert new_f["size"] == len(content)

    # show
    new_f = runner_remoteci.invoke(["job-show-file", job_id, "--file-id", new_f["id"]])[
        "file"
    ]
    assert new_f["size"] == len(content)

    # download
    runner_remoteci.invoke_raw(
        [
            "job-download-file",
            job_id,
            "--file-id",
            new_f["id"],
            "--target",
            td.strpath + "/my_file",
        ]
    )
    assert open(td.strpath + "/my_file", "rb").read() == content

    # list
    my_list = runner_remoteci.invoke(["job-list-file", job_id])["files"]
    assert len(my_list) == len(my_original_list) + 1
    assert my_list[0]["size"] == len(content)

    # delete
    runner_remoteci.invoke_raw(["job-delete-file", job_id, "--file-id", new_f["id"]])
    result = runner_remoteci.invoke_raw(
        ["job-show-file", job_id, "--file-id", new_f["id"]]
    )
    assert result.status_code == 404



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2017 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import csv
import json
import prettytable
from dciclient.v1.exceptions import BadParameter

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


def flatten(d, prefix=""):
    ret = []
    for k, v in d.items():
        p = k if not prefix else prefix + "." + k
        if isinstance(v, dict):
            ret += flatten(v, prefix=p)
        else:
            ret.append("%s=%s" % (p, v))
    return ret


def print_json(result_json):
    formatted_result = json.dumps(result_json, indent=4)
    print(formatted_result)


def print_csv(data, headers, skip_columns, delimiter=","):
    f = StringIO()
    data = _tablify_result(data)
    headers = headers or _find_headers_from_data(data)
    headers = _sort_headers(headers)
    headers = [i for i in headers if i not in skip_columns]
    data = [{k: v for k, v in d.items() if k not in skip_columns} for d in data]
    output = csv.DictWriter(f, headers, delimiter=delimiter)
    output.writerows(data)
    print(f.getvalue())


def _get_field(record, field_path):
    cur_field = field_path.pop(0)
    v = record.get(cur_field)
    if len(field_path):
        return _get_field(record[cur_field], field_path)
    else:
        return v


def _find_headers_from_data(data):
    """Return the header names from the data."""
    if isinstance(data, list):
        first_row = data[0] if len(data) else {}
    else:
        first_row = data
    return list(first_row.keys())


def _tablify_result(data):
    """Convert the JSON dict structure to a regular list."""
    if isinstance(data, dict):
        keys = [i for i in list(data.keys()) if i != "_meta"]
        if len(keys) == 1:
            data = data[keys[0]]
    if not isinstance(data, list):
        data = [data]
    return data


def _sort_headers(headers):
    """Ensure the column order is always the same."""
    headers = set(headers)
    default_order = ["id", "name", "etag", "created_at", "updated_at", "state", "data"]
    sorted_headers = []
    for i in default_order:
        if i not in headers:
            continue
        headers.remove(i)
        sorted_headers.append(i)
    sorted_headers += sorted(headers)
    return sorted_headers


def print_prettytable(data, headers=None, skip_columns=[]):
    data = _tablify_result(data)
    headers = headers or _find_headers_from_data(data)
    headers = _sort_headers(headers)
    headers = [i for i in headers if i not in skip_columns]
    table = prettytable.PrettyTable(headers)

    for record in data:
        row = []
        for item in headers:
            row.append(_get_field(record, field_path=item.split("/")))
        table.add_row(row)

    print(table)


def sanitize_kwargs(**kwargs):
    boolean_fields = ["active"]

    for k in list(kwargs.keys()):
        if kwargs[k] is None:
            if k in boolean_fields:
                kwargs[k] = bool(kwargs[k])
            else:
                del kwargs[k]
    try:
        kwargs["data"] = json.loads(kwargs["data"])
    except (KeyError, TypeError):
        pass

    return kwargs


def format_output(
    result, format, headers=None, success_code=(200, 201, 204), item=None, verbose=True
):

    skip_columns = []
    if not verbose:
        skip_columns = ["etag", "created_at", "updated_at", "data"]

    is_failure = False
    if hasattr(result, "json"):
        if result.status_code not in success_code:
            is_failure = True
        result = result.json()

    if format == "json" or is_failure:
        print_json(result)
        return

    # if our structure come with only one root key,
    # we can assume it's the item type.
    if not item and isinstance(result, dict):
        values = list(result.values())
        if len(values) == 1:
            result = values[0]
    to_display = result[item] if item else result
    if to_display:
        if format in ["csv", "tsv"]:
            delimiter = "\t" if format == "tsv" else ","
            print_csv(to_display, headers, skip_columns, delimiter=delimiter)
        else:
            print_prettytable(to_display, headers, skip_columns)


def validate_json(ctx, param, value):
    if value is None:
        return
    try:
        return json.loads(value)
    except ValueError:
        raise BadParameter("this option expects a valid JSON")


def active_string(value):
    return {None: None, True: "active", False: "inactive"}[value]



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pytest
import sqlalchemy
import sqlalchemy_utils.functions
import os
import passlib.apps as passlib_apps

import dci
import dci.app
import dci.dci_config
from dciclient.v1.api import context as api_context
from dciclient.v1.api import team as api_team
from dciclient.v1.api import user as api_user
from dciclient.v1.api import product as api_product
from dciclient.v1.api import remoteci as api_remoteci
from dciclient.v1.api import component as api_component
from dciclient.v1.api import file as api_file
from dciclient.v1.api import topic as api_topic
from dciclient.v1.api import job as api_job
from dciclient.v1.api import jobstate as api_jobstate
from dciclient.v1.api import test as api_test
from dciclient.v1.api import base as api_base
from dciclient.v1.shell_commands import runner as dci_runner
from dciclient.v1.shell_commands import cli
from tests.shell_commands import utils


class Mocked_store_engine(object):
    files = {}

    def __init__(self, conf):
        self.container = conf

    def delete(self, filename):
        del self.files[filename]

    def get(self, filename):
        fd = open("/tmp/swift/" + filename, "r")
        return [None, fd]

    # TODO(Goneri): should be dropped once
    # I9804796987e05417dcc20a1099b4a26db4f9f1f2 is merged
    def get_object(self, filename):
        _, fd = self.get(filename)
        return fd.read()

    def head(self, filename):
        return self.files[filename]

    def upload(self, filename, iterable, pseudo_folder=None, create_container=True):
        file_path = "/tmp/swift/" + filename
        if not os.path.isdir(os.path.dirname(file_path)):
            os.makedirs(os.path.dirname(file_path))
        with open(file_path, "wb") as fd:
            if hasattr(iterable, "read"):
                fd.write(iterable.read())
            else:
                fd.write(iterable)
        self.files[filename] = {
            "etag": "boby",
            "content-type": "application/octet-stream",
            "content-length": os.stat(file_path).st_size,
        }

    def build_file_path(self, root, middle, file_id):
        root = str(root)
        middle = str(middle)
        file_id = str(file_id)
        return "%s/%s/%s" % (root, middle, file_id)


@pytest.fixture(scope="session")
def engine(request):
    def mocked_get_store(conf):
        store = Mocked_store_engine(conf)
        return store

    dci.dci_config.get_store = mocked_get_store
    conf = dci.dci_config.generate_conf()
    db_uri = conf["SQLALCHEMY_DATABASE_URI"]

    engine = sqlalchemy.create_engine(db_uri)

    def del_db():
        if sqlalchemy_utils.functions.database_exists(db_uri):
            sqlalchemy_utils.functions.drop_database(db_uri)

    del_db()
    request.addfinalizer(del_db)
    sqlalchemy_utils.functions.create_database(db_uri)

    dci.db.models.metadata.create_all(engine)
    return engine


@pytest.fixture
def db_clean(request, engine):
    def fin():
        for table in reversed(dci.db.models.metadata.sorted_tables):
            engine.execute(table.delete())

    request.addfinalizer(fin)


@pytest.fixture(scope="session", autouse=True)
def memoize_password_hash():
    def memoize(func):
        cache = {}

        def helper(*args):
            if args in cache:
                return cache[args]
            else:
                value = func(*args)
                cache[args] = value
                return value

        return helper

    pwd_context = passlib_apps.custom_app_context
    pwd_context.verify = memoize(pwd_context.verify)
    pwd_context.encrypt = memoize(pwd_context.encrypt)


@pytest.fixture
def db_provisioning(db_clean, engine):
    with engine.begin() as conn:
        utils.provision(conn)


@pytest.fixture
def server(db_provisioning, engine):
    app = dci.app.create_app(dci.dci_config.generate_conf())
    app.testing = True
    app.engine = engine
    return app


def context_factory(
    server,
    db_provisioning,
    login,
    password,
    url="http://dciserver.com",
    user_agent=None,
):
    extras = {}
    if user_agent:
        extras["user_agent"] = user_agent
    test_context = api_context.DciContext(url, login, password, **extras)
    flask_adapter = utils.FlaskHTTPAdapter(server.test_client())
    test_context.session.mount(url, flask_adapter)
    return test_context


@pytest.fixture
def dci_context(server, db_provisioning):
    return context_factory(server, db_provisioning, "admin", "admin")


@pytest.fixture
def dci_context_user_admin(server, db_provisioning):
    return context_factory(server, db_provisioning, "user_admin", "user_admin")


@pytest.fixture
def dci_context_user(server, db_provisioning):
    return context_factory(server, db_provisioning, "user", "user")


@pytest.fixture
def dci_context_test_user(server, db_provisioning, test_user):
    return context_factory(server, db_provisioning, "foo", "pass")


@pytest.fixture
def dci_context_product_owner(server, db_provisioning):
    return context_factory(server, db_provisioning, "product_owner", "product_owner")


@pytest.fixture
def dci_context_other_user_agent(server, db_provisioning):
    return context_factory(
        server, db_provisioning, "admin", "admin", user_agent="myagent-0.1"
    )


@pytest.fixture
def dci_context_broken(server, db_provisioning):
    test_context = api_context.DciContext("http://no_where.com", "admin", "admin")
    test_context.last_job_id = 1
    return test_context


@pytest.fixture
def signature_context_factory(
    server,
    db_provisioning,
    client_id=None,
    api_secret=None,
    url="http://dciserver.com",
    user_agent=None,
):
    def f(
        server=server,
        db_provisioning=db_provisioning,
        client_id=client_id,
        api_secret=api_secret,
        url=url,
        user_agent=user_agent,
    ):
        extras = {}
        if user_agent:
            extras["user_agent"] = user_agent
        test_context = api_context.DciSignatureContext(url, client_id, api_secret)
        flask_adapter = utils.FlaskHTTPAdapter(server.test_client())
        test_context.session.mount(url, flask_adapter)
        return test_context

    return f


def runner_factory(context):
    def invoke(arguments):
        environment = {}
        args = cli.parse_arguments(arguments, environment)
        response = dci_runner.run(context, args)
        return response.json() if response else None

    def invoke_raw(arguments):
        environment = {}
        args = cli.parse_arguments(arguments, environment)
        return dci_runner.run(context, args)

    class Runner(object):
        pass

    runner = Runner()
    runner.invoke = invoke
    runner.invoke_raw = invoke_raw
    return runner


@pytest.fixture
def runner(dci_context):
    return runner_factory(dci_context)


@pytest.fixture
def runner_user_admin(dci_context_user_admin):
    return runner_factory(dci_context_user_admin)


@pytest.fixture
def runner_user(dci_context_user):
    return runner_factory(dci_context_user)


@pytest.fixture
def dci_context_remoteci(
    server, db_provisioning, remoteci_id, signature_context_factory, remoteci_api_secret
):
    return signature_context_factory(
        client_id=remoteci_id, api_secret=remoteci_api_secret
    )


@pytest.fixture
def runner_remoteci(dci_context_remoteci):
    return runner_factory(dci_context_remoteci)


@pytest.fixture
def team_id(dci_context):
    return api_team.create(dci_context, name="tname").json()["team"]["id"]


@pytest.fixture
def team_admin_id(dci_context):
    return api_team.list(dci_context, where="name:admin").json()["teams"][0]["id"]


@pytest.fixture
def team_user_id(dci_context):
    user = api_user.list(dci_context, where="name:user")
    return user.json()["users"][0]["team_id"]


@pytest.fixture
def product_id(dci_context, team_id):
    return api_product.create(dci_context, name="myproduct").json()["product"]["id"]


@pytest.fixture
def topic_id(dci_context, product_id):
    kwargs = {
        "name": "foo_topic",
        "component_types": ["type_1", "type_2"],
        "product_id": product_id,
        "export_control": False,
    }
    return api_topic.create(dci_context, **kwargs).json()["topic"]["id"]


@pytest.fixture
def test_id(dci_context, team_id):
    kwargs = {"name": "test_name", "team_id": team_id}
    return api_test.create(dci_context, **kwargs).json()["test"]["id"]


@pytest.fixture
def test_user_id(dci_context, team_user_id):
    kwargs = {"name": "test_user_name", "team_id": team_user_id}
    return api_test.create(dci_context, **kwargs).json()["test"]["id"]


@pytest.fixture
def remoteci_id(dci_context, team_user_id):
    kwargs = {
        "name": "remoteci",
        "team_id": team_user_id,
        "data": {"remoteci": "remoteci"},
    }
    rci = api_remoteci.create(dci_context, **kwargs).json()
    return rci["remoteci"]["id"]


@pytest.fixture
def remoteci_api_secret(dci_context, remoteci_id):
    rci = api_remoteci.get(dci_context, remoteci_id).json()
    return rci["remoteci"]["api_secret"]


@pytest.fixture
def component_id(dci_context, topic_id):
    kwargs = {
        "name": "component1",
        "type": "git_review",
        "data": {"component": "component"},
        "topic_id": topic_id,
    }

    component = api_component.create(dci_context, **kwargs).json()
    return component["component"]["id"]


@pytest.fixture
def components_ids(dci_context, topic_id):
    ids = []
    kwargs = {
        "name": "component1",
        "type": "type_1",
        "data": {"component": "component"},
        "topic_id": topic_id,
    }

    component = api_component.create(dci_context, **kwargs).json()
    ids.append(component["component"]["id"])
    kwargs["name"] = "component2"
    kwargs["type"] = "type_2"
    component = api_component.create(dci_context, **kwargs).json()
    ids.append(component["component"]["id"])
    return ids


@pytest.fixture
def job_factory(
    dci_context,
    dci_context_remoteci,
    team_user_id,
    remoteci_id,
    topic_id,
    components_ids,
):
    def create():
        job = api_job.schedule(dci_context_remoteci, topic_id).json()
        job_id = job["job"]["id"]
        api_file.create(
            dci_context_remoteci,
            name="res_junit.xml",
            content=JUNIT,
            mime="application/junit",
            job_id=job_id,
        )
        jobstate_id = api_jobstate.create(
            dci_context_remoteci, "pre-run", "starting", job_id
        ).json()["jobstate"]["id"]
        api_file.create(
            dci_context_remoteci,
            name="pre-run",
            content="pre-run ongoing",
            mime="plain/text",
            jobstate_id=jobstate_id,
        )
        api_jobstate.create(
            dci_context_remoteci, "running", "starting the build", job_id
        )
        return job

    JUNIT = """
    <testsuite errors="0" failures="0" name="pytest" skips="1"
               tests="3" time="46.050">
    <properties>
      <property name="x" value="y" />
      <property name="a" value="b" />
    </properties>
    <testcase classname="" file="test-requirements.txt"
              name="test-requirements.txt" time="0.0109479427338">
        <skipped message="all tests skipped by +SKIP option"
                 type="pytest.skip">Skipped for whatever reasons</skipped>
    </testcase>
    <testcase classname="tests.test_app" file="tests/test_app.py" line="26"
              name="test_cors_preflight" time="2.91562318802"/>
    <testcase classname="tests.test_app" file="tests/test_app.py" line="42"
              name="test_cors_headers" time="0.574683904648"/>
    </testsuite>"""

    api_topic.attach_team(dci_context, topic_id, team_user_id)
    return create


@pytest.fixture
def job_id(job_factory, components_ids):
    return job_factory()["job"]["id"]


@pytest.fixture
def job(job_factory):
    return job_factory()["job"]


@pytest.fixture
def file_id(dci_context, job_id):
    return api_job.list_files(dci_context, id=job_id).json()["files"][-1]["id"]


@pytest.fixture
def jobstate_id(dci_context, job_id):
    kwargs = {"job_id": job_id, "status": "running", "comment": "some comment"}
    jobstate = api_jobstate.create(dci_context, **kwargs).json()
    return jobstate["jobstate"]["id"]


@pytest.fixture
def test_user(runner, team_user_id):
    return runner.invoke(
        [
            "user-create",
            "--name",
            "foo",
            "--email",
            "foo@example.org",
            "--fullname",
            "Foo Bar",
            "--password",
            "pass",
            "--team-id",
            team_user_id,
        ]
    )["user"]


@pytest.fixture
def team_test(dci_context, team_id):
    test = api_test.create(dci_context, "sometest", team_id=team_id).json()
    return test["test"]


@pytest.fixture
def feeder(dci_context, team_id):
    kwargs = {"name": "feeder", "team_id": team_id, "state": "active"}
    feeder = api_base.create(dci_context, "feeders", **kwargs).json()
    return feeder["feeder"]



# -*- encoding: utf-8 -*-
#
# Copyright 2017 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base

RESOURCE = "feeders"


def create(context, name, team_id, data={}, state="active"):
    return base.create(
        context, RESOURCE, name=name, team_id=team_id, data=data, state=state
    )


def list(context, **kwargs):
    return base.list(context, RESOURCE, **kwargs)


def get(context, id, **kwargs):
    return base.get(context, RESOURCE, id=id, **kwargs)


def update(context, id, **kwargs):
    return base.update(context, RESOURCE, id=id, **kwargs)


def delete(context, id, etag):
    return base.delete(context, RESOURCE, id=id, etag=etag)


def reset_api_secret(context, id, etag):
    return base.update(context, RESOURCE, id="%s/api_secret" % id, etag=etag)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from dciclient.v1.shell_commands import analytic
from dciclient.v1.shell_commands import user
from dciclient.v1.shell_commands import team
from dciclient.v1.shell_commands import product
from dciclient.v1.shell_commands import feeder
from dciclient.v1.shell_commands import jobstate
from dciclient.v1.shell_commands import topic
from dciclient.v1.shell_commands import component
from dciclient.v1.shell_commands import file
from dciclient.v1.shell_commands import job
from dciclient.v1.shell_commands import test
from dciclient.v1.shell_commands import remoteci
from dciclient.v1.shell_commands import purge


command_function = {
    "analytic-list": analytic.list,
    "analytic-create": analytic.create,
    "analytic-show": analytic.show,
    "analytic-update": analytic.update,
    "user-list": user.list,
    "user-create": user.create,
    "user-show": user.show,
    "user-update": user.update,
    "user-delete": user.delete,
    "team-list": team.list,
    "team-create": team.create,
    "team-show": team.show,
    "team-update": team.update,
    "team-delete": team.delete,
    "product-list": product.list,
    "product-create": product.create,
    "product-show": product.show,
    "product-update": product.update,
    "product-delete": product.delete,
    "product-attach-team": product.attach_team,
    "product-detach-team": product.detach_team,
    "product-list-teams": product.list_teams,
    "feeder-list": feeder.list,
    "feeder-create": feeder.create,
    "feeder-show": feeder.show,
    "feeder-update": feeder.update,
    "feeder-delete": feeder.delete,
    "feeder-reset-api-secret": feeder.reset_api_secret,
    "jobstate-show": jobstate.show,
    "topic-list": topic.list,
    "topic-create": topic.create,
    "topic-show": topic.show,
    "topic-attach-team": topic.attach_team,
    "topic-unattach-team": topic.unattach_team,
    "topic-list-team": topic.list_team,
    "topic-update": topic.update,
    "topic-delete": topic.delete,
    "component-list": component.list,
    "component-create": component.create,
    "component-show": component.show,
    "component-attach-issue": component.attach_issue,
    "component-unattach-issue": component.unattach_issue,
    "component-list-issue": component.list_issues,
    "component-update": component.update,
    "component-delete": component.delete,
    "component-file-list": component.file_list,
    "component-file-upload": component.file_upload,
    "component-file-show": component.file_show,
    "component-file-download": component.file_download,
    "component-file-delete": component.file_delete,
    "file-list": file.list,
    "file-show": file.show,
    "file-delete": file.delete,
    "job-list": job.list,
    "job-show": job.show,
    "job-delete": job.delete,
    "job-results": job.list_results,
    "job-attach-issue": job.attach_issue,
    "job-unattach-issue": job.unattach_issue,
    "job-list-issue": job.list_issues,
    "job-output": job.output,
    "job-list-test": job.list_tests,
    "job-add-tag": job.add_tag,
    "job-delete-tag": job.delete_tag,
    "job-list-tags": job.list_tags,
    "job-upload-file": job.file_upload,
    "job-download-file": job.file_download,
    "job-show-file": job.file_show,
    "job-list-file": job.file_list,
    "job-delete-file": job.file_delete,
    "test-list": test.list,
    "test-create": test.create,
    "test-update": test.update,
    "test-delete": test.delete,
    "test-show": test.show,
    "remoteci-list": remoteci.list,
    "remoteci-create": remoteci.create,
    "remoteci-update": remoteci.update,
    "remoteci-delete": remoteci.delete,
    "remoteci-show": remoteci.show,
    "remoteci-get-data": remoteci.get_data,
    "remoteci-attach-test": remoteci.attach_test,
    "remoteci-unattach-test": remoteci.unattach_test,
    "remoteci-list-test": remoteci.list_test,
    "remoteci-attach-user": remoteci.attach_user,
    "remoteci-unattach-user": remoteci.unattach_user,
    "remoteci-list-user": remoteci.list_user,
    "remoteci-reset-api-secret": remoteci.reset_api_secret,
    "remoteci-refresh-keys": remoteci.refresh_keys,
    "purge": purge.purge,
}


def run(context, args):
    return command_function[args.command](context, args)



# -*- encoding: utf-8 -*-
#
# Copyright 2017 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

RESOURCE = "identity"


def get(context):
    uri = "%s/%s" % (context.dci_cs_api, RESOURCE)
    return context.session.get(uri)


def my_team_id(context):
    """Asks the control-server for the team_id of the currently
    authenticated resource.
    """
    return get(context).json()["identity"]["team_id"]



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.utils import active_string
from dciclient.v1.utils import validate_json
from dciclient.v1.api import test


def list(context, args):
    params = {k: getattr(args, k) for k in ["sort", "limit", "offset", "where"]}
    return test.list(context, **params)


def create(context, args):
    data = validate_json(context, "data", args.data)
    state = active_string(args.state)
    return test.create(context, name=args.name, data=data, state=state)


def update(context, args):
    params = {k: getattr(args, k) for k in ["id", "name", "etag", "data", "state"]}
    params["state"] = active_string(params["state"])
    return test.update(context, **params)


def delete(context, args):
    return test.delete(context, id=args.id)


def show(context, args):
    return test.get(context, id=args.id)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base


RESOURCE = "users"


def create(context, name, password, team_id, email, fullname, state="active"):
    return base.create(
        context,
        RESOURCE,
        name=name,
        password=password,
        team_id=team_id,
        state=state,
        email=email,
        fullname=fullname,
    )


def list(context, **kwargs):
    return base.list(context, RESOURCE, **kwargs)


def get(context, id, **kwargs):
    return base.get(context, RESOURCE, id=id, **kwargs)


def get_current_user(context):
    uri = "%s/%s/me" % (context.dci_cs_api, RESOURCE)
    return context.session.get(uri)


def update(
    context,
    id,
    etag,
    name=None,
    password=None,
    team_id=None,
    state=None,
    email=None,
    fullname=None,
):
    return base.update(
        context,
        RESOURCE,
        id=id,
        etag=etag,
        name=name,
        password=password,
        team_id=team_id,
        state=state,
        email=email,
        fullname=fullname,
    )


def delete(context, id, etag):
    return base.delete(context, RESOURCE, id=id, etag=etag)



# -*- encoding: utf-8 -*-
#
# Copyright 2017 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals


def test_success_create_basic(runner):
    product = runner.invoke(["product-create", "--name", "myproduct"])["product"]
    assert product["name"] == "myproduct"


def test_success_create_full(runner):
    product = runner.invoke(
        [
            "product-create",
            "--name",
            "myproduct",
            "--label",
            "MYPRODUCT",
            "--description",
            "myproduct",
            "--active",
        ]
    )["product"]
    assert product["name"] == "myproduct"
    assert product["label"] == "MYPRODUCT"
    assert product["description"] == "myproduct"
    assert product["state"] == "active"


def test_create_inactive(runner):
    product = runner.invoke(["product-create", "--name", "myproduct", "--no-active"])[
        "product"
    ]
    assert product["state"] == "inactive"


def test_list(runner):
    products_number = len(runner.invoke(["product-list"])["products"])

    runner.invoke(["product-create", "--name", "foo"])
    runner.invoke(["product-create", "--name", "bar"])

    products_new_number = len(runner.invoke(["product-list"])["products"])

    assert products_new_number == products_number + 2


def test_fail_create_unauthorized_user_admin(runner_user_admin):
    product = runner_user_admin.invoke_raw(["product-create", "--name", "foo"])
    assert product.status_code == 401


def test_fail_create_unauthorized_user(runner_user):
    product = runner_user.invoke_raw(["product-create", "--name", "foo"])
    assert product.status_code == 401


def test_success_update(runner):
    product = runner.invoke(
        ["product-create", "--name", "foo", "--description", "foo_desc"]
    )["product"]

    result = runner.invoke(
        [
            "product-update",
            product["id"],
            "--etag",
            product["etag"],
            "--name",
            "bar",
            "--description",
            "bar_desc",
        ]
    )

    assert result["product"]["id"] == product["id"]
    assert result["product"]["name"] == "bar"
    assert result["product"]["description"] == "bar_desc"


def test_update_active(runner):
    product = runner.invoke(["product-create", "--name", "myproduct"])["product"]
    assert product["state"] == "active"

    result = runner.invoke(
        ["product-update", product["id"], "--etag", product["etag"], "--no-active"]
    )

    assert result["product"]["id"] == product["id"]
    assert result["product"]["state"] == "inactive"

    result = runner.invoke(
        [
            "product-update",
            product["id"],
            "--etag",
            result["product"]["etag"],
            "--name",
            "foobar",
        ]
    )

    assert result["product"]["state"] == "inactive"

    result = runner.invoke(
        [
            "product-update",
            product["id"],
            "--etag",
            result["product"]["etag"],
            "--active",
        ]
    )

    assert result["product"]["state"] == "active"


def test_delete(runner):
    product = runner.invoke(["product-create", "--name", "foo"])["product"]

    result = runner.invoke_raw(
        ["product-delete", product["id"], "--etag", product["etag"]]
    )

    assert result.status_code == 204

    result = runner.invoke_raw(["product-show", product["id"]])

    assert result.status_code == 404


def test_show(runner):
    product = runner.invoke(["product-create", "--name", "foo"])["product"]

    product = runner.invoke(["product-show", product["id"]])["product"]

    assert product["name"] == "foo"


def test_list_teams_empty(runner, product_id, team_id):
    no_teams = runner.invoke(["product-list-teams", product_id])
    assert no_teams["_meta"]["count"] == 0
    assert len(no_teams["teams"]) == 0


def test_attach_team_and_list(runner, product_id, team_id):
    no_teams = runner.invoke(["product-list-teams", product_id])
    assert no_teams["_meta"]["count"] == 0
    assert len(no_teams["teams"]) == 0

    attached = runner.invoke(["product-attach-team", product_id, "--team-id", team_id])

    assert attached["product_id"] == product_id
    assert attached["team_id"] == team_id

    one_team = runner.invoke(["product-list-teams", product_id])
    assert one_team["_meta"]["count"] == 1
    assert len(one_team["teams"]) == 1
    assert one_team["teams"][0]["id"] == team_id


def test_attach_detach_team_and_list(runner, product_id, team_id):
    no_teams = runner.invoke(["product-list-teams", product_id])
    assert no_teams["_meta"]["count"] == 0
    assert len(no_teams["teams"]) == 0

    attached = runner.invoke(["product-attach-team", product_id, "--team-id", team_id])

    assert attached["product_id"] == product_id
    assert attached["team_id"] == team_id

    one_team = runner.invoke(["product-list-teams", product_id])

    assert one_team["_meta"]["count"] == 1
    assert len(one_team["teams"]) == 1
    assert one_team["teams"][0]["id"] == team_id

    runner.invoke_raw(["product-detach-team", product_id, "--team-id", team_id])
    no_teams_again = runner.invoke(["product-list-teams", product_id])
    assert no_teams_again["_meta"]["count"] == 0
    assert len(no_teams_again["teams"]) == 0



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2017 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base
from dciclient.v1.api import remoteci
from dciclient.v1.api import topic
from dciclient.v1 import utils
from dciclient.v1.api.tag import add_tag_to_resource, delete_tag_from_resource


RESOURCE = "jobs"


def create(
    context, topic_id, team_id=None, components=None, comment=None, previous_job_id=None
):
    job = base.create(
        context,
        RESOURCE,
        topic_id=topic_id,
        team_id=team_id,
        components=components,
        comment=comment,
        previous_job_id=previous_job_id,
    )
    context.last_job_id = job.json()["job"]["id"]
    return job


def update(context, **kwargs):
    return base.update(context, RESOURCE, **kwargs)


def list(context, **kwargs):
    return base.list(context, RESOURCE, **kwargs)


def schedule(context, topic_id, components=None):
    uri = "%s/%s/schedule" % (context.dci_cs_api, RESOURCE)
    data = {"topic_id": topic_id, "components_ids": components}
    data = utils.sanitize_kwargs(**data)
    r = context.session.post(uri, json=data)
    if r.status_code == 201:
        context.last_job_id = r.json()["job"]["id"]
    return r


def job_update(context, job_id):
    uri = "%s/%s/%s/update" % (context.dci_cs_api, RESOURCE, job_id)
    r = context.session.post(uri)
    if r.status_code == 201:
        context.last_job_id = r.json()["job"]["id"]
    return r


def upgrade(context, job_id):
    uri = "%s/%s/upgrade" % (context.dci_cs_api, RESOURCE)
    data = {"job_id": job_id}
    r = context.session.post(uri, json=data)
    if r.status_code == 201:
        context.last_job_id = r.json()["job"]["id"]
    return r


def get(context, id, **kwargs):
    return base.get(context, RESOURCE, id=id, **kwargs)


def list_results(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id, subresource="results", **kwargs)


def iter(context, **kwargs):
    return base.iter(context, RESOURCE, **kwargs)


def get_components(context, id):
    uri = "%s/%s/%s/components" % (context.dci_cs_api, RESOURCE, id)
    return context.session.get(uri)


def delete(context, id, etag):
    return base.delete(context, RESOURCE, id=id, etag=etag)


def list_files(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id, subresource="files", **kwargs)


def list_files_iter(context, id, **kwargs):
    return base.iter(context, RESOURCE, id=id, subresource="files", **kwargs)


def list_issues(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id, subresource="issues", **kwargs)


def attach_issue(context, id, url):
    uri = "%s/%s/%s/issues" % (context.dci_cs_api, RESOURCE, id)
    data = {"url": url}
    return context.session.post(uri, json=data)


def unattach_issue(context, id, issue_id):
    return base.delete(
        context, RESOURCE, id, subresource="issues", subresource_id=issue_id
    )


def list_jobstates(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id, subresource="jobstates", **kwargs)


def list_tests(context, id, **kwargs):
    j = base.get(context, RESOURCE, id=id, **kwargs).json()["job"]
    result = {"tests": []}
    result["tests"] += topic.list_tests(context, j["topic_id"]).json()["tests"]
    result["tests"] += remoteci.list_tests(context, j["remoteci_id"]).json()["tests"]
    return result


def list_tags(context, id):
    return base.list(context, RESOURCE, id=id, subresource="tags")


def add_tag(context, id, name):
    return add_tag_to_resource(context, RESOURCE, id, name)


def delete_tag(context, id, tag_id):
    return delete_tag_from_resource(context, RESOURCE, id, tag_id)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import io
import requests.adapters
import requests.models
import requests.utils

from dci import auth
from dci.db import models


class FlaskHTTPAdapter(requests.adapters.HTTPAdapter):
    def __init__(self, flask_client):
        super(FlaskHTTPAdapter, self).__init__(pool_connections=1)
        self.client = flask_client

    def init_poolmanager(self, *args, **kwargs):
        pass

    def build_response(self, req, resp):
        response = requests.models.Response()
        response.status_code = resp.status_code

        response.headers = resp.headers
        response.encoding = (
            requests.utils.get_encoding_from_headers(resp.headers) or "utf-8"
        )
        response.raw = io.BytesIO(resp.data)
        response.request = req
        response.connection = self
        return response

    def close(self):
        pass

    def send(
        self, request, stream=False, timeout=None, verify=True, cert=None, proxies=None
    ):

        content_type = request.headers.pop("Content-Type", None)
        content_length = request.headers.pop("Content-Length", None)
        # https://github.com/pallets/werkzeug/pull/1011
        if hasattr(request.body, "read"):
            request.body = request.body.read()

        response = self.client.open(
            request.path_url,
            method=request.method.lower(),
            data=request.body,
            headers=dict(request.headers),
            content_type=content_type,
            content_length=content_length,
        )
        return self.build_response(request, response)


def generate_componenttype(client):
    return client.post("/componenttypes", {"name": "my_component_type"}).json()


def generate_job(client, topic_id):
    componenttype = generate_componenttype(client)

    client.post("/tests", {"name": "my_test"})
    component = client.post(
        "/components",
        {
            "name": "my_component",
            "componenttype_id": componenttype["id"],
            "sha": "some_sha",
            "canonical_project_name": "my_project",
        },
    ).json()

    team = client.post("/teams", {"name": "my_team"}).json()

    remoteci = client.post("/remotecis", {"team_id": team["id"]}).json()

    job = client.post(
        "/jobs",
        {
            "topic_id": topic_id,
            "team_id": team["id"],
            "remoteci_id": remoteci["id"],
            "components": [component["id"]],
        },
    ).json()

    return job


def provision(db_conn):
    def db_insert(model_item, return_pk=True, **kwargs):
        query = model_item.insert().values(**kwargs)
        if return_pk:
            return db_conn.execute(query).inserted_primary_key[0]
        else:
            db_conn.execute(query)

    # Create teams
    team_admin_id = db_insert(models.TEAMS, name="admin")
    db_insert(models.TEAMS, name="Red Hat")
    db_insert(models.TEAMS, name="EPM")
    team_product_id = db_insert(models.TEAMS, name="product")
    team_user_id = db_insert(models.TEAMS, name="user")

    # Create users
    user_pw_hash = auth.hash_password("user")
    u_id = db_insert(
        models.USERS,
        name="user",
        sso_username="user",
        password=user_pw_hash,
        fullname="User",
        email="user@example.org",
        team_id=team_user_id,
    )

    db_insert(
        models.JOIN_USERS_TEAMS, return_pk=False, user_id=u_id, team_id=team_user_id
    )

    user_no_team_pw_hash = auth.hash_password("user_no_team")
    u_id = db_insert(
        models.USERS,
        name="user_no_team",
        sso_username="user_no_team",
        password=user_no_team_pw_hash,
        fullname="User No Team",
        email="user_no_team@example.org",
        team_id=None,
    )

    db_insert(models.JOIN_USERS_TEAMS, return_pk=False, user_id=u_id, team_id=None)

    product_owner_pw_hash = auth.hash_password("product_owner")
    u_id = db_insert(
        models.USERS,
        name="product_owner",
        sso_username="product_owner",
        password=product_owner_pw_hash,
        fullname="Product Owner",
        email="product_ownern@example.org",
        team_id=team_product_id,
    )

    db_insert(
        models.JOIN_USERS_TEAMS, return_pk=False, user_id=u_id, team_id=team_product_id
    )

    admin_pw_hash = auth.hash_password("admin")
    u_id = db_insert(
        models.USERS,
        name="admin",
        sso_username="admin",
        password=admin_pw_hash,
        fullname="Admin",
        email="admin@example.org",
        team_id=team_admin_id,
    )

    db_insert(
        models.JOIN_USERS_TEAMS, return_pk=False, user_id=u_id, team_id=team_admin_id
    )

    # Create a product
    db_insert(
        models.PRODUCTS,
        name="Awesome product",
        label="AWSM",
        description="My Awesome product",
    )



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import user
from dciclient.v1.utils import active_string


def list(context, args):
    params = {k: getattr(args, k) for k in ["sort", "limit", "offset", "where"]}
    return user.list(context, **params)


def create(context, args):
    params = {
        k: getattr(args, k)
        for k in ["name", "password", "email", "team_id", "fullname", "state"]
    }
    params["fullname"] = params["fullname"] or params["name"]
    params["state"] = active_string(params["state"])
    return user.create(context, **params)


def show(context, args):
    return user.get(context, args.id)


def update(context, args):
    params = {
        k: getattr(args, k)
        for k in [
            "name",
            "password",
            "email",
            "team_id",
            "fullname",
            "state",
            "id",
            "etag",
        ]
    }
    params["fullname"] = params["fullname"] or params["name"]
    params["state"] = active_string(params["state"])
    return user.update(context, **params)


def delete(context, args):
    return user.delete(context, args.id, args.etag)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2017 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import component


def test_success_download_component_file_returns_http_response(
    dci_context, component_id, tmpdir
):
    """Ensure an http response is retrieved."""

    tmp_file = tmpdir.mkdir("data").join("content")
    tmp_file.write("DISTRIBUTED-CI")

    component.file_upload(dci_context, component_id, tmp_file.strpath)

    file_id = component.file_list(dci_context, component_id).json()["component_files"][
        0
    ]["id"]

    res = component.file_download(dci_context, component_id, file_id, tmp_file.strpath)

    assert res.status_code == 200


def test_add_tag(dci_context, component_id):
    res = component.add_tag(dci_context, component_id, "tag 1")
    assert res.status_code == 201


def test_delete_tag(dci_context, component_id):
    tag = component.add_tag(dci_context, component_id, "tag 1").json()["tag"]
    res = component.delete_tag(dci_context, component_id, tag["id"])
    assert res.status_code == 204



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.utils import active_string
from dciclient.v1.utils import validate_json


from dciclient.v1.api import identity
from dciclient.v1.api import remoteci


def list(context, args):
    params = {k: getattr(args, k) for k in ["sort", "limit", "offset", "where"]}
    return remoteci.list(context, **params)


def create(context, args):
    data = validate_json(context, "data", args.data)
    state = active_string(args.state)
    team_id = args.team_id or identity.my_team_id(context)
    return remoteci.create(
        context, name=args.name, team_id=team_id, data=data, state=state
    )


def update(context, args):
    data = validate_json(context, "data", args.data)
    state = active_string(args.state)
    return remoteci.update(
        context,
        id=args.id,
        etag=args.etag,
        name=args.name,
        team_id=args.team_id,
        data=data,
        state=state,
    )


def delete(context, args):
    return remoteci.delete(context, id=args.id, etag=args.etag)


def show(context, args):
    return remoteci.get(context, id=args.id)


def get_data(context, args):
    keys = args.keys.split(",") if args.keys else args.keys
    return remoteci.get_data(context, id=args.id, keys=keys)


def attach_test(context, args):
    return remoteci.add_test(context, id=args.id, test_id=args.test_id)


def list_test(context, args):
    params = {k: getattr(args, k) for k in ["id", "sort", "limit", "offset", "where"]}
    return remoteci.list_tests(context, **params)


def unattach_test(context, args):
    return remoteci.remove_test(context, id=args.id, test_id=args.test_id)


def reset_api_secret(context, args):
    return remoteci.reset_api_secret(context, id=args.id, etag=args.etag)


def refresh_keys(context, args):
    return remoteci.refresh_keys(context, id=args.id, etag=args.etag)


def attach_user(context, args):
    return remoteci.add_user(context, id=args.id, user_id=args.user_id)


def list_user(context, args):
    params = {k: getattr(args, k) for k in ["id", "sort", "limit", "offset", "where"]}
    return remoteci.list_users(context, **params)


def unattach_user(context, args):
    return remoteci.remove_user(context, id=args.id, user_id=args.user_id)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import remoteci
from dciclient.v1.api import team

import mock


def test_list(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]
    runner.invoke(["remoteci-create", "--name", "foo", "--team-id", team["id"]])
    runner.invoke(["remoteci-create", "--name", "bar", "--team-id", team["id"]])
    remotecis = runner.invoke(["remoteci-list"])["remotecis"]

    assert len(remotecis) == 2
    assert remotecis[0]["name"] == "bar"
    assert remotecis[1]["name"] == "foo"


def test_create(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]
    remoteci = runner.invoke(
        ["remoteci-create", "--name", "foo", "--team-id", team["id"]]
    )["remoteci"]
    assert remoteci["name"] == "foo"
    assert remoteci["state"] == "active"


def test_create_inactive(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]
    remoteci = runner.invoke(
        ["remoteci-create", "--name", "foo", "--team-id", team["id"], "--no-active"]
    )["remoteci"]
    assert remoteci["state"] == "inactive"


def test_update(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]
    remoteci = runner.invoke(
        ["remoteci-create", "--name", "foo", "--team-id", team["id"]]
    )["remoteci"]

    result = runner.invoke(
        [
            "remoteci-update",
            remoteci["id"],
            "--etag",
            remoteci["etag"],
            "--name",
            "bar",
            "--no-active",
        ]
    )

    assert result["remoteci"]["id"] == remoteci["id"]
    assert result["remoteci"]["name"] == "bar"


def test_update_active(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]
    remoteci = runner.invoke(
        ["remoteci-create", "--name", "foo", "--team-id", team["id"]]
    )["remoteci"]

    assert remoteci["state"] == "active"

    result = runner.invoke(
        ["remoteci-update", remoteci["id"], "--etag", remoteci["etag"], "--no-active"]
    )

    assert result["remoteci"]["id"] == remoteci["id"]
    assert result["remoteci"]["state"] == "inactive"

    result = runner.invoke(
        [
            "remoteci-update",
            remoteci["id"],
            "--etag",
            result["remoteci"]["etag"],
            "--name",
            "foobar",
        ]
    )

    assert result["remoteci"]["id"] == remoteci["id"]
    assert result["remoteci"]["state"] == "inactive"

    result = runner.invoke(
        [
            "remoteci-update",
            remoteci["id"],
            "--etag",
            result["remoteci"]["etag"],
            "--active",
        ]
    )

    assert result["remoteci"]["state"] == "active"


def test_delete(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]

    remoteci = runner.invoke(
        ["remoteci-create", "--name", "foo", "--team-id", team["id"]]
    )["remoteci"]

    result = runner.invoke_raw(
        ["remoteci-delete", remoteci["id"], "--etag", remoteci["etag"]]
    )

    assert result.status_code == 204


def test_show(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]
    remoteci = runner.invoke(
        ["remoteci-create", "--name", "foo", "--team-id", team["id"]]
    )["remoteci"]

    remoteci = runner.invoke(["remoteci-show", remoteci["id"]])["remoteci"]

    assert remoteci["name"] == "foo"


def test_get_data(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]
    remoteci = runner.invoke(
        ["remoteci-create", "--name", "foo", "--team-id", team["id"]]
    )["remoteci"]

    result = runner.invoke(["remoteci-get-data", remoteci["id"]])
    assert result == {}
    runner.invoke(
        [
            "remoteci-update",
            remoteci["id"],
            "--etag",
            remoteci["etag"],
            "--data",
            '{"foo": "bar"}',
        ]
    )
    result = runner.invoke(["remoteci-get-data", remoteci["id"]])
    assert result == {"foo": "bar"}


def test_get_data_missing_key(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]
    remoteci = runner.invoke(
        ["remoteci-create", "--name", "foo", "--team-id", team["id"]]
    )["remoteci"]

    result = runner.invoke(["remoteci-get-data", remoteci["id"], "--keys", "missing"])
    assert result == {}


def test_embed(dci_context):
    team_id = team.create(dci_context, name="teama").json()["team"]["id"]

    rci = remoteci.create(dci_context, name="boa", team_id=team_id).json()
    rci_id = rci["remoteci"]["id"]

    rci_with_embed = remoteci.get(dci_context, id=rci_id, embed="team").json()
    embed_team_id = rci_with_embed["remoteci"]["team"]["id"]

    assert team_id == embed_team_id


def test_where_on_list(runner, team_id):
    runner.invoke(["remoteci-create", "--name", "bar1", "--team-id", team_id])
    runner.invoke(["remoteci-create", "--name", "bar2", "--team-id", team_id])
    assert (
        runner.invoke(["remoteci-list", "--where", "name:bar1"])["_meta"]["count"] == 1
    )


def test_refresh_remoteci_keys(runner, remoteci_id):
    with mock.patch("requests.sessions.Session.put") as post_mock:
        post_mock.return_value = '{"key": "XXX", "cert": "XXX" }'
        runner.invoke_raw(["remoteci-refresh-keys", remoteci_id, "--etag", "XX"])
        url = "http://dciserver.com/api/v1/remotecis/%s/keys" % remoteci_id
        post_mock.assert_called_once_with(
            url, headers={"If-match": "XX"}, json={}, timeout=600
        )



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pytest
from dciclient.v1.exceptions import BadParameter


def test_create_list(runner):
    runner.invoke(["test-create", "--name", "foo"])
    runner.invoke(["test-create", "--name", "bar"])
    tests = runner.invoke(["test-list"])["tests"]
    assert len(tests) == 2
    assert tests[0]["name"] == "bar"
    assert tests[1]["name"] == "foo"


def test_create_inactive(runner):
    test = runner.invoke(["test-create", "--name", "foo", "--no-active"])["test"]
    assert test["state"] == "inactive"


def test_create_data(runner):
    test = runner.invoke(["test-create", "--name", "foo", "--data", '{"Foo": 2}'])[
        "test"
    ]
    assert test["name"] == "foo"


def test_create_bad_data(runner):
    with pytest.raises(BadParameter):
        runner.invoke_raw(["test-create", "--name", "foo", "--data", "{Foo: 2}"])


def test_update_active(runner):
    test = runner.invoke(["test-create", "--name", "foo", "--data", '{"Foo": 2}'])[
        "test"
    ]

    assert test["state"] == "active"

    result = runner.invoke(
        ["test-update", test["id"], "--etag", test["etag"], "--no-active"]
    )

    assert result["test"]["id"] == test["id"]
    assert result["test"]["state"] == "inactive"

    result = runner.invoke(
        [
            "test-update",
            test["id"],
            "--etag",
            result["test"]["etag"],
            "--name",
            "foobar",
        ]
    )

    assert result["test"]["id"] == test["id"]
    assert result["test"]["state"] == "inactive"
    assert result["test"]["name"] == "foobar"

    result = runner.invoke(
        ["test-update", test["id"], "--etag", result["test"]["etag"], "--active"]
    )

    assert result["test"]["state"] == "active"


def test_delete(runner):
    test = runner.invoke(["test-create", "--name", "foo"])["test"]
    tests = runner.invoke(["test-list"])["tests"]
    len_tests = len(tests)

    runner.invoke_raw(["test-delete", test["id"]])
    tests = runner.invoke(["test-list"])["tests"]
    len_tests2 = len(tests)

    assert len_tests2 == (len_tests - 1)


def test_show(runner):
    test = runner.invoke(["test-create", "--name", "foo"])["test"]

    test = runner.invoke(["test-show", test["id"]])["test"]

    assert test["name"] == "foo"



# -*- coding: utf-8 -*-
#
# Copyright (C) Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import jobs_events
from dciclient.v1.api import jobstate


def test_jobs_events_create(dci_context, job_id):

    js = jobstate.create(dci_context, "success", "lol", job_id)
    assert js.status_code == 201
    all_je = jobs_events.list(dci_context, 0)
    assert all_je.status_code == 200
    all_je_data = all_je.json()
    assert len(all_je_data["jobs_events"]) > 0


def test_jobs_events_delete_from_sequence(dci_context, job_id):

    js = jobstate.create(dci_context, "success", "lol", job_id)
    assert js.status_code == 201
    all_je = jobs_events.list(dci_context, 0)
    assert all_je.status_code == 200
    all_je_data = all_je.json()
    assert len(all_je_data["jobs_events"]) > 0

    jobs_events.delete(dci_context, 0)

    all_je = jobs_events.list(dci_context, 0)
    assert all_je.status_code == 200
    all_je_data = all_je.json()
    assert len(all_je_data["jobs_events"]) == 0


def test_jobs_event_sequence(dci_context):

    je = jobs_events.get_sequence(dci_context)
    assert je.status_code == 200
    je_data = je.json()["sequence"]
    assert "sequence" in je_data
    assert "etag" in je_data

    je_put = jobs_events.update_sequence(dci_context, je_data["etag"], 1234)
    assert je_put.status_code == 204

    je = jobs_events.get_sequence(dci_context)
    assert je.status_code == 200
    assert je.json()["sequence"]["sequence"] == 1234



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import requests

from dciauth.request import AuthRequest
from dciauth.signature import Signature
from dciclient.v1.api import job
import tests.shell_commands.utils as utils


def make_blank_session(server):
    s = requests.Session()
    flask_adapter = utils.FlaskHTTPAdapter(server.test_client())
    s.mount("http://dciserver.com", flask_adapter)
    return s


def test_get_job_with_no_auth_fails(server, job_id):
    s = make_blank_session(server)
    r = s.get("http://dciserver.com/api/v1/jobs/%s" % job_id)
    assert r.status_code == 401


def test_get_job_with_signature_succeeds(
    server, job_id, remoteci_id, remoteci_api_secret
):
    request = AuthRequest(endpoint="/api/v1/jobs/%s" % job_id,)
    headers = Signature(request).generate_headers(
        client_type="remoteci", client_id=remoteci_id, secret=remoteci_api_secret
    )
    s = make_blank_session(server)
    r = s.get("http://dciserver.com/api/v1/jobs/%s" % job_id, headers=headers)
    assert r.status_code == 200


def test_get_job_with_remoteci_context_succeeds(dci_context_remoteci, job_id):
    context = dci_context_remoteci
    r = job.get(context, job_id)
    assert r.status_code == 200


# todo: will be reactivated once in the server we will differentiate
# agent from users
def loltest_get_job_with_feeder_context_fails(
    feeder, signature_context_factory, job_id
):
    context = signature_context_factory(
        client_id="feeder/%s" % feeder["id"], api_secret=feeder["api_secret"]
    )
    r = job.get(context, job_id)
    assert r.status_code == 404


def test_get_job_with_bad_type_context_fails(feeder, signature_context_factory, job_id):
    context = signature_context_factory(
        client_id="bad_type/%s" % feeder["id"], api_secret=feeder["api_secret"]
    )
    r = job.get(context, job_id)
    assert r.status_code == 401


def test_server_url_with_trailing_slash(
    remoteci_id, remoteci_api_secret, signature_context_factory, job_id
):
    dci_context = signature_context_factory(
        client_id=remoteci_id,
        api_secret=remoteci_api_secret,
        url="http://dciserver.com/",
    )

    r = job.get(dci_context, job_id)
    assert r.status_code == 200



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.printer import print_response


def test_printer(capsys, runner, team_id):
    runner.invoke_raw(
        [
            "user-create",
            "--name",
            "foo",
            "--email",
            "foo@example.org",
            "--password",
            "pass",
            "--team-id",
            team_id,
        ]
    )
    print_response(
        runner.invoke_raw(["user-list"]), format="table", verbose=True,
    )
    captured = capsys.readouterr()
    assert "etag" in captured.out


def test_printer_verbose(capsys, runner, team_id):
    runner.invoke_raw(
        [
            "user-create",
            "--name",
            "foo",
            "--email",
            "foo@example.org",
            "--password",
            "pass",
            "--team-id",
            team_id,
        ]
    )
    print_response(
        runner.invoke_raw(["user-list"]), format="table", verbose=False,
    )
    captured = capsys.readouterr()
    assert "etag" not in captured.out


def test_printer_delete(capsys, runner, team_id):
    user = runner.invoke(
        [
            "user-create",
            "--name",
            "todelete",
            "--email",
            "todelete@example.org",
            "--password",
            "pass",
            "--team-id",
            team_id,
        ]
    )["user"]
    result = runner.invoke_raw(["user-delete", user["id"], "--etag", user["etag"]])
    print_response(result, format="table", verbose=False)
    captured = capsys.readouterr()
    assert "resource deleted" in captured.out



# -*- encoding: utf-8 -*-
#
# Copyright Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base


RESOURCE = "tags"


def create(context, name):
    return base.create(context, RESOURCE, name=name)


def add_tag_to_resource(context, resource, id, name):
    uri = "%s/%s/%s/tags" % (context.dci_cs_api, resource, id)
    return context.session.post(uri, json={"name": name})


def list(context):
    return base.get(context, RESOURCE)


def delete_tag_from_resource(context, resource, id, tag_id):
    return base.delete(context, resource, id, subresource="tags", subresource_id=tag_id)


def delete(context, id):
    return base.delete(context, RESOURCE, id)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


def test_create_user(runner, test_user, team_user_id):
    assert test_user["name"] == "foo"
    assert test_user["fullname"] == "Foo Bar"
    assert test_user["email"] == "foo@example.org"
    assert test_user["team_id"] == team_user_id


def test_create_admin(runner, team_id):
    user = runner.invoke(
        [
            "user-create",
            "--name",
            "foo",
            "--email",
            "foo@example.org",
            "--password",
            "pass",
            "--team-id",
            team_id,
        ]
    )["user"]
    assert user["name"] == "foo"
    assert user["fullname"] == "foo"
    assert user["email"] == "foo@example.org"
    assert user["team_id"] == team_id


def test_create_super_admin(runner, team_id):
    user = runner.invoke(
        [
            "user-create",
            "--name",
            "foo",
            "--email",
            "foo@example.org",
            "--password",
            "pass",
            "--team-id",
            team_id,
        ]
    )["user"]
    assert user["name"] == "foo"
    assert user["fullname"] == "foo"
    assert user["email"] == "foo@example.org"
    assert user["team_id"] == team_id


def test_create_inactive(runner, team_id):
    user = runner.invoke(
        [
            "user-create",
            "--name",
            "foo",
            "--password",
            "pass",
            "--email",
            "foo@example.org",
            "--team-id",
            team_id,
            "--no-active",
        ]
    )["user"]
    assert user["state"] == "inactive"


def test_list(runner, team_id):
    users_cnt = len(runner.invoke(["user-list"])["users"])
    runner.invoke(
        [
            "user-create",
            "--name",
            "bar",
            "--email",
            "bar@example.org",
            "--password",
            "pass",
            "--team-id",
            team_id,
        ]
    )
    new_users_cnt = len(runner.invoke(["user-list"])["users"])
    assert new_users_cnt == users_cnt + 1


def test_update(runner, test_user):
    runner.invoke(
        [
            "user-update",
            test_user["id"],
            "--etag",
            test_user["etag"],
            "--name",
            "bar",
            "--email",
            "bar@example.org",
            "--fullname",
            "Barry White",
        ]
    )
    user = runner.invoke(["user-show", test_user["id"]])["user"]

    assert user["name"] == "bar"
    assert user["fullname"] == "Barry White"
    assert user["email"] == "bar@example.org"


def test_update_team_id(runner, test_user, team_user_id, team_id):
    user = runner.invoke(["user-show", test_user["id"]])["user"]
    assert user["team_id"] == team_user_id

    runner.invoke(
        [
            "user-update",
            test_user["id"],
            "--etag",
            test_user["etag"],
            "--team-id",
            team_id,
        ]
    )
    user = runner.invoke(["user-show", test_user["id"]])["user"]
    assert user["team_id"] == team_id


def test_update_active(runner, test_user, team_id):
    assert test_user["state"] == "active"

    result = runner.invoke(
        ["user-update", test_user["id"], "--etag", test_user["etag"], "--no-active"]
    )

    assert result["user"]["id"] == test_user["id"]
    assert result["user"]["state"] == "inactive"

    result = runner.invoke(
        [
            "user-update",
            test_user["id"],
            "--etag",
            result["user"]["etag"],
            "--name",
            "foobar",
        ]
    )

    assert result["user"]["id"] == test_user["id"]
    assert result["user"]["state"] == "inactive"
    assert result["user"]["name"] == "foobar"

    result = runner.invoke(
        ["user-update", test_user["id"], "--etag", result["user"]["etag"], "--active"]
    )

    assert result["user"]["state"] == "active"


def test_delete(runner, test_user, team_id):
    result = runner.invoke_raw(
        ["user-delete", test_user["id"], "--etag", test_user["etag"]]
    )
    assert result.status_code == 204


def test_show(runner, test_user, team_id):
    user = runner.invoke(["user-show", test_user["id"]])["user"]

    assert user["name"] == test_user["name"]


def test_where_on_list(runner, test_user, team_id):
    runner.invoke(
        [
            "user-create",
            "--name",
            "foo2",
            "--email",
            "foo2@example.org",
            "--password",
            "pass",
            "--team-id",
            team_id,
        ]
    )
    runner.invoke(
        [
            "user-create",
            "--name",
            "foo3",
            "--email",
            "foo3@example.org",
            "--password",
            "pass",
            "--team-id",
            team_id,
        ]
    )
    users_cnt = len(runner.invoke(["user-list"])["users"])
    assert runner.invoke(["user-list"])["_meta"]["count"] == users_cnt
    assert runner.invoke(["user-list", "--where", "name:foo"])["_meta"]["count"] == 1



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2017 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


def test_purge_wrong_resource(runner):
    result = runner.invoke_raw(["purge", "--force", "--resource", "wrongresource"])
    assert "Unknown resource have been specified:" in result
    assert "wrongresource" in result


def test_purge_success_authorized_admin(runner):
    result = runner.invoke_raw(["purge"])
    assert result == {}


def test_purge_fail_unauthorized_user(runner_user):
    result = runner_user.invoke_raw(["purge"])
    assert result.status_code == 401


def test_purge_fail_unauthorized_user_admin(runner_user_admin):
    result = runner_user_admin.invoke_raw(["purge"])
    assert result.status_code == 401


def test_purge_noop(runner, remoteci_id, product_id):
    runner.invoke(["topic-create", "--name", "osp", "--product-id", product_id])
    runner.invoke(["topic-create", "--name", "osp2", "--product-id", product_id])
    topics = runner.invoke(["topic-list"])["topics"]
    topic_id = topics[0]["id"]
    assert len(topics) == 2

    runner.invoke_raw(["topic-delete", topic_id])
    topics = runner.invoke(["topic-list"])["topics"]
    assert len(topics) == 1

    purge_res = runner.invoke_raw(["purge", "--resource", "topics"])
    assert purge_res["topics"]["topics"][0]["id"] == topic_id
    assert purge_res["topics"]["topics"][0]["state"] == "archived"

    runner.invoke_raw(["purge", "--resource", "topics", "--force"])

    purge_res = runner.invoke_raw(["purge", "--resource", "topics"])
    assert len(purge_res) == 0

    topics = runner.invoke(["topic-list"])["topics"]
    assert len(topics) == 1



# -*- encoding: utf-8 -*-
#
# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base


RESOURCE = "topics"


def create(
    context,
    name,
    component_types,
    label=None,
    state="active",
    product_id=None,
    next_topic_id=None,
    next_topic=None,
    data=None,
    export_control=False,
):
    return base.create(
        context,
        RESOURCE,
        name=name,
        label=label,
        state=state,
        component_types=component_types,
        product_id=product_id,
        next_topic_id=next_topic_id,
        next_topic=next_topic,
        data=data,
        export_control=export_control,
    )


def list(context, **kwargs):
    return base.list(context, RESOURCE, **kwargs)


def get(context, id, **kwargs):
    return base.get(context, RESOURCE, id=id, **kwargs)


def update(
    context,
    id,
    etag,
    name=None,
    component_types=None,
    label=None,
    next_topic_id=None,
    next_topic=None,
    state=None,
    product_id=None,
    data=None,
    export_control=None,
):
    return base.update(
        context,
        RESOURCE,
        id=id,
        etag=etag,
        name=name,
        component_types=component_types,
        label=label,
        next_topic_id=next_topic_id,
        state=state,
        product_id=product_id,
        next_topic=next_topic,
        data=data,
        export_control=export_control,
    )


def delete(context, id):
    return base.delete(context, RESOURCE, id=id)


def attach_team(context, id, team_id):
    uri = "%s/%s/%s/teams" % (context.dci_cs_api, RESOURCE, id)
    return context.session.post(uri, json={"team_id": team_id})


def unattach_team(context, id, team_id):
    return base.delete(
        context, RESOURCE, id, subresource="teams", subresource_id=team_id
    )


def list_teams(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id, subresource="teams", **kwargs)


def list_components(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id, subresource="components", **kwargs)


def list_tests(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id, subresource="tests", **kwargs)


def add_test(context, id, test_id):
    uri = "%s/%s/%s/tests" % (context.dci_cs_api, RESOURCE, id)
    return context.session.post(uri, json={"test_id": test_id})


def remove_test(context, id, test_id):
    return base.delete(
        context, RESOURCE, id, subresource="tests", subresource_id=test_id
    )


def get_jobs_from_components(context, id, component_id, **kwargs):
    uri = "%s/%s/%s/components/%s/jobs" % (
        context.dci_cs_api,
        RESOURCE,
        id,
        component_id,
    )
    return context.session.get(uri, **kwargs)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.shell import main
from mock import patch
import sys


@patch("dciclient.shell.print_response")
@patch("dciclient.shell.run")
def test_build_valid_remoteci_context(
    mock_run, mock_printer, remoteci_id, remoteci_api_secret
):
    test_args = [
        "dcictl",
        "--dci-client-id",
        remoteci_id,
        "--dci-api-secret",
        remoteci_api_secret,
        "component-list",
        "--topic-id",
        "id",
    ]
    with patch.object(sys, "argv", test_args):
        main()
    assert mock_run.called
    assert mock_run.call_args_list[0][0][0] is not None
    assert mock_printer.called



# -*- coding: utf-8 -*-
#
# Copyright (C) 2017 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import product


def test_success_create_product_authorized(dci_context):
    """Create a product."""

    products_original_cnt = product.list(dci_context).json()["_meta"]["count"]
    product.create(dci_context, "OpenStack")
    products_test = product.list(dci_context).json()
    products_test_cnt = products_test["_meta"]["count"]

    assert products_test_cnt == products_original_cnt + 1
    assert "OpenStack" in [prod["name"] for prod in products_test["products"]]


def test_failure_create_product_unauthorized(dci_context_user_admin):
    """Fail to create a product with an unauthorized identity."""

    result = product.create(dci_context_user_admin, "OpenStack")

    assert result.status_code == 401


def test_success_list_all_products(dci_context, product_id):
    """List all products."""

    products_original_cnt = product.list(dci_context).json()["_meta"]["count"]
    product.create(dci_context, "OpenStack")
    products_test = product.list(dci_context).json()
    products_test_cnt = products_test["_meta"]["count"]

    assert products_test_cnt == products_original_cnt + 1
    assert product_id in [prod["id"] for prod in products_test["products"]]
    assert "OpenStack" in [prod["name"] for prod in products_test["products"]]


def test_success_show_product(dci_context, product_id):
    """Retrieve a specific product."""

    product_to_retrieve = product.get(dci_context, product_id).json()

    assert product_id == product_to_retrieve["product"]["id"]


def test_success_update_product_authorized(dci_context, product_id):
    """Update a specific product."""

    product_to_retrieve = product.get(dci_context, product_id).json()
    product_etag = product_to_retrieve["product"]["etag"]

    updated_params = {"description": "This is a new description"}
    product.update(dci_context, product_id, etag=product_etag, **updated_params)

    product_to_retrieve = product.get(dci_context, product_id).json()

    assert "This is a new description" == product_to_retrieve["product"]["description"]


def test_failure_update_product_unauthorized(
    dci_context, dci_context_user_admin, product_id
):
    """Fail to update a product with an unauthorized identity."""

    product_to_retrieve = product.get(dci_context, product_id).json()
    product_etag = product_to_retrieve["product"]["etag"]

    updated_params = {"description": "This is a new description"}
    result = product.update(
        dci_context_user_admin, product_id, etag=product_etag, **updated_params
    )

    assert result.status_code == 401


def test_success_delete_product_authorized(dci_context, product_id):
    """Delete a product that belongs."""

    product_to_retrieve = product.get(dci_context, product_id).json()
    product_etag = product_to_retrieve["product"]["etag"]

    products_original_cnt = product.list(dci_context).json()["_meta"]["count"]
    product.delete(dci_context, product_id, etag=product_etag)
    products_test = product.list(dci_context).json()
    products_test_cnt = products_test["_meta"]["count"]

    assert products_test_cnt == products_original_cnt - 1
    assert product_id not in [prod["id"] for prod in products_test["products"]]


def test_failure_delete_product_unauthorized(
    dci_context, dci_context_user_admin, product_id
):
    """Fail to delete a product that belongs."""

    product_to_retrieve = product.get(dci_context, product_id).json()
    product_etag = product_to_retrieve["product"]["etag"]

    result = product.delete(dci_context_user_admin, product_id, etag=product_etag)

    assert result.status_code == 401



# -*- encoding: utf-8 -*-
#
# Copyright 2017 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


def test_list(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]
    runner.invoke(["feeder-create", "--name", "foo", "--team-id", team["id"]])
    runner.invoke(["feeder-create", "--name", "bar", "--team-id", team["id"]])
    feeders = runner.invoke(["feeder-list"])["feeders"]

    assert len(feeders) == 2
    assert feeders[0]["name"] == "bar"
    assert feeders[1]["name"] == "foo"


def test_create(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]
    feeder = runner.invoke(["feeder-create", "--name", "foo", "--team-id", team["id"]])[
        "feeder"
    ]
    assert feeder["name"] == "foo"
    assert feeder["state"] == "active"


def test_create_inactive(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]
    feeder = runner.invoke(
        ["feeder-create", "--name", "foo", "--team-id", team["id"], "--no-active"]
    )["feeder"]
    assert feeder["state"] == "inactive"


def test_update(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]
    feeder = runner.invoke(["feeder-create", "--name", "foo", "--team-id", team["id"]])[
        "feeder"
    ]

    assert feeder["state"] == "active"

    result = runner.invoke(
        [
            "feeder-update",
            feeder["id"],
            "--etag",
            feeder["etag"],
            "--name",
            "bar",
            "--no-active",
        ]
    )

    assert result["feeder"]["id"] == feeder["id"]
    assert result["feeder"]["state"] == "inactive"


def test_delete(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]

    feeder = runner.invoke(["feeder-create", "--name", "foo", "--team-id", team["id"]])[
        "feeder"
    ]

    result = runner.invoke_raw(
        ["feeder-delete", feeder["id"], "--etag", feeder["etag"]]
    )

    assert result.status_code == 204


def test_show(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]
    feeder = runner.invoke(["feeder-create", "--name", "foo", "--team-id", team["id"]])[
        "feeder"
    ]

    feeder = runner.invoke(["feeder-show", feeder["id"]])["feeder"]

    assert feeder["name"] == "foo"



# -*- encoding: utf-8 -*-
#
# Copyright 2017 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from dciclient.v1.utils import active_string
from dciclient.v1.api import product


def list(context, args):
    params = {k: getattr(args, k) for k in ["sort", "limit", "offset", "where"]}
    return product.list(context, **params)


def create(context, args):
    params = {k: getattr(args, k) for k in ["name", "label", "description", "state"]}
    params["state"] = active_string(params["state"])
    return product.create(context, **params)


def update(context, args):
    params = {
        k: getattr(args, k)
        for k in ["id", "etag", "name", "label", "description", "state"]
    }
    params["state"] = active_string(params["state"])
    return product.update(context, **params)


def delete(context, args):
    return product.delete(context, args.id, etag=args.etag)


def show(context, args):
    return product.get(context, args.id)


def attach_team(context, args):
    params = {k: getattr(args, k) for k in ["id", "team_id"]}
    return product.attach_team(context, **params)


def detach_team(context, args):
    params = {k: getattr(args, k) for k in ["id", "team_id"]}
    return product.detach_team(context, **params)


def list_teams(context, args):
    params = {k: getattr(args, k) for k in ["id", "sort", "limit", "offset", "where"]}
    return product.list_teams(context, **params)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base


RESOURCE = "tests"


def create(context, name, data={}, state="active"):
    return base.create(context, RESOURCE, name=name, data=data, state=state)


def list(context, **kwargs):
    return base.list(context, RESOURCE, **kwargs)


def get(context, id, **kwargs):
    return base.get(context, RESOURCE, id=id, **kwargs)


def delete(context, id):
    return base.delete(context, RESOURCE, id=id)


def update(context, id, etag, name=None, data=None, state=None):
    return base.update(
        context, RESOURCE, id=id, etag=etag, name=name, data=data, state=state
    )



# -*- encoding: utf-8 -*-
#
# Copyright Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pytest

from mock import patch
from dciclient.v1.shell_commands.cli import parse_arguments


def test_parse_arguments_user_list_defaults():
    args = parse_arguments(["user-list"])
    assert args.command == "user-list"
    assert args.sort == "-created_at"
    assert args.limit == 50
    assert args.offset == 0
    assert args.where is None
    assert args.verbose is False


@pytest.mark.parametrize(
    "params",
    [
        ["user-create"],
        ["user-create", "--name", "foo"],
        ["user-create", "--name", "foo", "--password", "bar"],
    ],
)
@patch("sys.exit")
def test_parse_arguments_user_create_no_required_args_exit(exit_function, params):
    parse_arguments(params)
    assert exit_function.called


@patch("sys.exit")
def test_parse_arguments_user_create_required_args_no_exit(exit_function):
    args = parse_arguments(
        ["user-create", "--name", "foo", "--password", "bar", "--email", "foo@foo.bar"]
    )
    assert args.command == "user-create"
    assert args.name == "foo"
    assert args.password == "bar"
    assert args.email == "foo@foo.bar"
    assert not exit_function.called


def test_parse_arguments_user_create_boolean_flags():
    args = parse_arguments(
        [
            "user-create",
            "--name",
            "foo",
            "--password",
            "bar",
            "--email",
            "foo@foo.bar",
            "--active",
        ]
    )
    assert args.state
    args = parse_arguments(
        [
            "user-create",
            "--name",
            "foo",
            "--password",
            "bar",
            "--email",
            "foo@foo.bar",
            "--no-active",
        ]
    )
    assert args.state is False


def test_parse_arguments_user_create_active_default():
    args = parse_arguments(
        ["user-create", "--name", "foo", "--password", "bar", "--email", "foo@foo.bar"]
    )
    assert args.state is True


def test_parse_arguments_user_update():
    args = parse_arguments(["user-update", "u1", "--etag", "e1", "--name", "new_name"])
    assert args.command == "user-update"
    assert args.id == "u1"
    assert args.etag == "e1"
    assert args.name == "new_name"


def test_parse_arguments_user_delete():
    args = parse_arguments(["user-delete", "u1", "--etag", "e1"])
    assert args.command == "user-delete"
    assert args.id == "u1"
    assert args.etag == "e1"



# -*- encoding: utf-8 -*-
#
# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import component
from dciclient.v1.api import job
from dciclient.v1.api import topic


def test_job_create_as_remoteci(
    dci_context, dci_context_remoteci, components_ids, topic_id, team_user_id, job_id
):
    topic.attach_team(dci_context, topic_id, team_user_id)

    j = job.create(
        dci_context_remoteci,
        topic_id=topic_id,
        team_id=team_user_id,
        components=components_ids,
        previous_job_id=job_id,
    ).json()
    assert j["job"]["id"]
    assert j["job"]["previous_job_id"] == job_id


def test_job_put(dci_context, job_id):
    new_comment = "foo"
    j = job.get(dci_context, job_id).json()["job"]
    assert j["comment"] is None
    job.update(dci_context, id=job_id, etag=j["etag"], comment=new_comment)
    r = job.get(dci_context, job_id)
    j = r.json()["job"]
    assert j["comment"] == new_comment


def test_job_update(dci_context, job_id):
    r = job.job_update(dci_context, job_id=job_id)
    assert r.status_code == 201
    assert r.json()["job"]["update_previous_job_id"] == job_id


def test_job_upgraded(dci_context, job_id, topic_id, product_id):
    old = topic.get(dci_context, id=topic_id).json()["topic"]
    new = topic.create(dci_context, "bar_topic", ["type_1"], product_id=product_id)
    t = new.json()["topic"]
    component.create(dci_context, "bar_component", "type_1", t["id"])
    topic.update(dci_context, id=topic_id, etag=old["etag"], next_topic_id=t["id"])
    r = job.upgrade(dci_context, job_id=job_id)
    assert r.status_code == 201
    assert r.json()["job"]["previous_job_id"] == job_id



# -*- encoding: utf-8 -*-
#
# Copyright Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1 import utils

from dciclient.v1.api import base


RESOURCE = "analytics"


def create(context, job_id, name, type, url, data):
    data = utils.sanitize_kwargs(name=name, type=type, url=url, data=data)
    uri = "%s/jobs/%s/%s" % (context.dci_cs_api, job_id, RESOURCE)
    return context.session.post(uri, timeout=base.HTTP_TIMEOUT, json=data)


def get(context, id, job_id):
    uri = "%s/jobs/%s/%s/%s" % (context.dci_cs_api, job_id, RESOURCE, id)
    return context.session.get(uri, timeout=base.HTTP_TIMEOUT)


def list(context, job_id):
    uri = "%s/jobs/%s/%s" % (context.dci_cs_api, job_id, RESOURCE)
    return context.session.get(uri, timeout=base.HTTP_TIMEOUT)


def update(context, id, job_id, etag, name, type, url, data):
    put_data = utils.sanitize_kwargs(name=name, type=type, url=url, data=data)
    uri = "%s/jobs/%s/%s/%s" % (context.dci_cs_api, job_id, RESOURCE, id)
    return context.session.put(
        uri, timeout=base.HTTP_TIMEOUT, json=put_data, headers={"If-match": etag}
    )


def delete(context, id, job_id):
    uri = "%s/jobs/%s/%s/%s" % (context.dci_cs_api, job_id, RESOURCE, id)
    return context.session.delete(uri, timeout=base.HTTP_TIMEOUT)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import os
import sys
from dciclient.v1.shell_commands.cli import parse_arguments
from dciclient.v1.api import context as dci_context
from dciclient.v1.shell_commands.runner import run
from dciclient.printer import print_response


def main():
    args = parse_arguments(sys.argv[1:], os.environ)
    dci_cs_url = args.dci_cs_url
    dci_login = args.dci_login
    dci_password = args.dci_password
    context = None
    if dci_login is not None and dci_password is not None:
        context = dci_context.build_dci_context(
            dci_login=dci_login, dci_password=dci_password, dci_cs_url=dci_cs_url
        )
    sso_url = args.sso_url
    sso_username = args.sso_username
    sso_password = args.sso_password
    sso_token = args.sso_token
    refresh_sso_token = args.refresh_sso_token
    if (
        sso_url is not None and sso_username is not None and sso_password is not None
    ) or sso_token is not None:
        context = dci_context.build_sso_context(
            dci_cs_url,
            sso_url,
            sso_username,
            sso_password,
            sso_token,
            refresh=refresh_sso_token,
        )
    dci_client_id = args.dci_client_id
    dci_api_secret = args.dci_api_secret
    if dci_client_id is not None and dci_api_secret is not None:
        context = dci_context.build_signature_context(
            dci_cs_url=dci_cs_url,
            dci_client_id=dci_client_id,
            dci_api_secret=dci_api_secret,
        )
    if not context:
        print("No credentials provided.")
        sys.exit(1)
    response = run(context, args)
    print_response(response, args.format, args.verbose)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2017 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base


def purge(context, args):
    resources = [
        "components",
        "topics",
        "tests",
        "teams",
        "feeders",
        "remotecis",
        "jobs",
        "files",
        "users",
        "products",
    ]

    l_resources = resources if args.resource is None else args.resource.split(",")

    wrong_resources = [res for res in l_resources if res not in resources]
    test_auth = base.purge(context, "users", **{"force": False})

    if len(wrong_resources) > 0:
        msg = "Unknown resource have been specified: %s" % wrong_resources
        return msg

    elif test_auth.status_code == 401:
        return test_auth

    else:
        purged = {}
        if args.force:
            # If in force mode. First we retrieve the number of items to be
            # purged and then we purge them. This allows to presents meaningful
            # informations to the user that used this command.

            for res in l_resources:
                item_purged = base.purge(context, res, **{"force": False}).json()[
                    "_meta"
                ]["count"]
                if (
                    item_purged
                    and base.purge(context, res, **{"force": True}).status_code == 204
                ):
                    purged[res] = "%s item(s) purged" % item_purged
            return purged
        else:
            # If not in force mode. The various endpoints are queried for the
            # informations about the resources to be purged and displayed.
            for res in l_resources:
                resource_to_delete = base.purge(context, res, **{"force": args.force})
                if resource_to_delete.json()["_meta"]["count"] > 0:
                    purged[res] = resource_to_delete.json()
            return purged



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base


RESOURCE = "remotecis"


def create(context, name, team_id, data={}, state="active"):
    return base.create(
        context, RESOURCE, name=name, team_id=team_id, data=data, state=state
    )


def list(context, **kwargs):
    return base.list(context, RESOURCE, **kwargs)


def get(context, id, **kwargs):
    return base.get(context, RESOURCE, id=id, **kwargs)


def get_data(context, id, keys=None):
    return base.get_data(context, RESOURCE, id=id, keys=keys)


def update(
    context, id, etag, name=None, team_id=None, data=None, state=None,
):
    return base.update(
        context,
        RESOURCE,
        id=id,
        etag=etag,
        name=name,
        team_id=team_id,
        data=data,
        state=state,
    )


def delete(context, id, etag):
    return base.delete(context, RESOURCE, id=id, etag=etag)


def add_test(context, id, test_id):
    uri = "%s/%s/%s/tests" % (context.dci_cs_api, RESOURCE, id)
    return context.session.post(uri, json={"test_id": test_id})


def list_tests(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id, subresource="tests", **kwargs)


def remove_test(context, id, test_id):
    return base.delete(
        context, RESOURCE, id, subresource="tests", subresource_id=test_id
    )


def reset_api_secret(context, id, etag):
    return base.update(context, RESOURCE, id="%s/api_secret" % id, etag=etag)


def refresh_keys(context, id, etag):
    return base.update(context, RESOURCE, id="%s/keys" % id, etag=etag)


def add_user(context, id, user_id):
    uri = "%s/%s/%s/users" % (context.dci_cs_api, RESOURCE, id)
    return context.session.post(uri, json={"user_id": user_id})


def list_users(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id, subresource="users", **kwargs)


def remove_user(context, id, user_id):
    return base.delete(
        context, RESOURCE, id, subresource="users", subresource_id=user_id
    )



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals


def test_list(runner):
    teams = runner.invoke(["team-list"])["teams"]
    current_nb_teams = len(teams)
    runner.invoke(["team-create", "--name", "foo"])
    runner.invoke(["team-create", "--name", "bar"])
    teams = runner.invoke(["team-list"])["teams"]
    assert (current_nb_teams + 2) == len(teams)


def test_create(runner, team_admin_id):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]
    assert team["name"] == "foo"


def test_create_with_country_and_email(runner):
    team = runner.invoke(["team-create", "--name", "foo", "--country", "FR"])["team"]
    assert team["name"] == "foo"
    assert team["country"] == "FR"


def test_create_inactive(runner):
    team = runner.invoke(["team-create", "--name", "foo", "--no-active"])["team"]
    assert team["state"] == "inactive"


def test_create_fail_unauthorized_user_admin(runner_user_admin):
    team = runner_user_admin.invoke_raw(["team-create", "--name", "foo"])
    assert team.status_code == 401


def test_create_fail_unauthorized_user(runner_user):
    team = runner_user.invoke_raw(["team-create", "--name", "foo"])
    assert team.status_code == 401


def test_update(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]

    result = runner.invoke(
        [
            "team-update",
            team["id"],
            "--etag",
            team["etag"],
            "--name",
            "bar",
            "--country",
            "JP",
        ]
    )

    assert result["team"]["id"] == team["id"]
    assert result["team"]["name"] == "bar"
    assert result["team"]["country"] == "JP"


def test_update_active(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]
    assert team["state"] == "active"

    result = runner.invoke(
        ["team-update", team["id"], "--etag", team["etag"], "--no-active"]
    )

    assert result["team"]["id"] == team["id"]
    assert result["team"]["state"] == "inactive"

    result = runner.invoke(
        [
            "team-update",
            team["id"],
            "--etag",
            result["team"]["etag"],
            "--name",
            "foobar",
        ]
    )

    assert result["team"]["id"] == team["id"]
    assert result["team"]["state"] == "inactive"

    result = runner.invoke(
        ["team-update", team["id"], "--etag", result["team"]["etag"], "--active"]
    )

    assert result["team"]["state"] == "active"


def test_update_team_external(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]
    runner.invoke(["team-update", team["id"], "--etag", team["etag"], "--no-external"])
    team = runner.invoke(["team-show", team["id"]])["team"]

    assert team["external"] is False


def test_delete(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]

    result = runner.invoke_raw(["team-delete", team["id"], "--etag", team["etag"]])

    assert result.status_code == 204


def test_show(runner):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]

    team = runner.invoke(["team-show", team["id"]])["team"]

    assert team["name"] == "foo"


def test_where_on_list(runner):
    runner.invoke(["team-create", "--name", "foobar42"])

    assert (
        runner.invoke(["team-list", "--where", "name:foobar42"])["_meta"]["count"] == 1
    )
